# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Simplified BSD License (see simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import re

from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.common.text.converters import to_native

Client = None
try:
    from clickhouse_driver import Client
    from clickhouse_driver.errors import ServerException
    from clickhouse_driver import __version__ as driver_version
    HAS_DB_DRIVER = True
except ImportError:
    HAS_DB_DRIVER = False

VALID_IDENTIFIER_PATTERN = re.compile(r'^[^`\\]+$')


def client_common_argument_spec():
    """
    Return a dictionary with connection options.

    The options are commonly used by many modules.
    """
    return dict(
        login_host=dict(type='str', default='localhost'),
        login_port=dict(type='int', default=None),
        login_db=dict(type='str', default=None),
        login_user=dict(type='str', default=None),
        login_password=dict(type='str', default=None, no_log=True),
        client_kwargs=dict(type='dict', default={}),
        success_on=dict(type='list', elements='int', default=[497]),
    )


def get_main_conn_kwargs(module):
    """Retrieves main connection arguments values and translates
    them into corresponding clickhouse_driver.Client() arguments.

    Returns a dictionary of arguments with values to pass to Client().
    """
    main_conn_kwargs = {}
    main_conn_kwargs['host'] = module.params['login_host']  # Has a default value
    if module.params['login_port']:
        main_conn_kwargs['port'] = module.params['login_port']
    if module.params['login_db']:
        main_conn_kwargs['database'] = module.params['login_db']
    if module.params['login_user']:
        main_conn_kwargs['user'] = module.params['login_user']
    if module.params['login_password']:
        main_conn_kwargs['password'] = module.params['login_password']
    return main_conn_kwargs


def check_clickhouse_driver(module):
    """Checks if the driver is present.

    Informs user if no driver and fails.
    """
    if not HAS_DB_DRIVER:
        module.fail_json(msg=missing_required_lib('clickhouse_driver'))


def version_clickhouse_driver():
    """
    Returns the current version of clickhouse_driver.
    """
    return driver_version


def connect_to_db_via_client(module, main_conn_kwargs, client_kwargs):
    """Connects to DB using the Client() class.

    Returns Client() object.
    """
    try:
        # Merge the kwargs as Python 2 would throw an error
        # when unpacking them separately to Client()
        client_kwargs.update(main_conn_kwargs)
        client = Client(**client_kwargs)
        client.connection.connect()
    except Exception as e:
        module.fail_json(msg="Failed to connect to database: %s" % to_native(e))

    # Display warning about using unsuporrted server version.
    server_version = get_server_version(module, client)
    if server_version['year'] < 24 or server_version['year'] == 24 and server_version['feature'] < 8:
        module.warn("Used server version is not activately maintained with this collection. Some features may not work properly.")
    return client


def execute_query(module, client, query, execute_kwargs=None, set_settings=None, custom_message=None):
    """Execute query.

    Returns rows returned in response.

    set_settings - The list of settings that need to be set before executing the request.
    """
    # Some modules do not pass this argument
    if execute_kwargs is None:
        execute_kwargs = {}

    if set_settings is None:
        set_settings = {}

    try:
        if len(set_settings) != 0:
            for setting in set_settings:
                client.execute("SET %s = '%s'" % (setting, set_settings[setting]))
        result = client.execute(query, **execute_kwargs)
    except ServerException as e:
        if e.code in module.params['success_on']:
            module.exit_json(changed=False, msg="Code %i defined as success." % e.code, custom_message=custom_message)
        module.fail_json(
            msg="Failed to execute query.",
            exception=to_native(e),
            code=e.code,
            message=e.message,
            query=query,
            custom_message=custom_message
        )
    return result


def get_server_version(module, client):
    """Get server version.

    Returns a dictionary with server version.
    """
    result = client.connection.server_info.version_tuple()

    version = {}

    version["year"] = result[0]
    version["feature"] = result[1]
    version["maintenance"] = result[2]

    return version


def get_on_cluster_clause(module, cluster):
    if not cluster:
        return ""
    validate_identifier(module, cluster, "cluster name")
    return f" ON CLUSTER `{cluster}`"


def validate_identifier(module, name, context="identifier"):
    if not name:
        module.fail_json(msg=f"{context.capitalize()} cannot be empty")
    elif not VALID_IDENTIFIER_PATTERN.match(name):
        module.fail_json(msg=f"Invalid {context}: '{name}'")
    return name


def normalize_db_table(module, client, database, table):
    """We want to make sure target is correct.
    When passed without db, default for session will apply and it can break idempotency.
    Ex:
        db, table → `db`.`table`
        None, table    → `default`.`table`
        db, *     → `db`.*
        db, None → `db`.*
    """
    if not database:
        query = "SELECT currentDatabase()"
        database = execute_query(module, client, query)[0][0]
    else:
        validate_identifier(module, database)
    if not table or table == '*':
        return f"`{database}`.*"
    else:
        validate_identifier(module, table)
    return f"`{database}`.`{table}`"



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_set_default_administrator_settings
short_description: Set default administrator settings.
description:
  - Set default administrator settings.
  - This command is available only after logging in to the System Data domain.
  - All operations are performed over Web Services API.
  - Available from R82.10 management version.
version_added: "6.7.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  authentication_method:
    description:
      - Authentication method for new administrator.
    type: str
    choices: ['undefined', 'check point password', 'os password', 'securid', 'radius', 'tacacs', 'ad authentication', 'api key', 'identity provider']
  expiration_type:
    description:
      - Expiration type for new administrator.
    type: str
    choices: ['never', 'expiration date', 'expiration period']
  expiration_date:
    description:
      - Expiration date for new administrator in YYYY-MM-DD format. <font color="red">Required only when</font> 'expiration-type' is set to 'expiration date'.
    type: str
  expiration_period:
    description:
      - Expiration period for new administrator. <font color="red">Required only when</font> 'expiration-type' is set to 'expiration period'.
      - Valid values are between 1 and 99.
    type: int
  expiration_period_time_units:
    description:
      - Expiration period time units for new administrator. <font color="red">Required only when</font> 'expiration-type' is set to 'expiration period'.
    type: str
    choices: ['days', 'months', 'years']
  indicate_expiration_in_admin_view:
    description:
      - Indicates whether to notify administrator about expiration.
    type: bool
  notify_expiration_to_admin:
    description:
      - Indicates whether to show 'about to expire' indication in administrator view.
    type: bool
  days_to_indicate_expiration_in_admin_view:
    description:
      - Number of days in advanced to show 'about to expire' indication in administrator view.
      - Valid values are between 1 and 99.
    type: int
  days_to_notify_expiration_to_admin:
    description:
      - Number of days in advanced to notify administrator about expiration.
      - Valid values are between 1 and 99.
    type: int
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-default-administrator-settings
  cp_mgmt_set_default_administrator_settings:
    days_to_notify_expiration_to_admin: 5
    expiration_date: '2025-06-23'
    expiration_type: expiration date
    indicate_expiration_in_admin_view: false
    notify_expiration_to_admin: true
"""

RETURN = """
cp_mgmt_set_default_administrator_settings:
  description: The checkpoint set-default-administrator-settings output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        authentication_method=dict(type='str', choices=['undefined', 'check point password',
                                                        'os password', 'securid', 'radius', 'tacacs', 'ad authentication', 'api key', 'identity provider']),
        expiration_type=dict(type='str', choices=['never', 'expiration date', 'expiration period']),
        expiration_date=dict(type='str'),
        expiration_period=dict(type='int'),
        expiration_period_time_units=dict(type='str', choices=['days', 'months', 'years']),
        indicate_expiration_in_admin_view=dict(type='bool'),
        notify_expiration_to_admin=dict(type='bool'),
        days_to_indicate_expiration_in_admin_view=dict(type='int'),
        days_to_notify_expiration_to_admin=dict(type='int')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "set-default-administrator-settings"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# Copyright (c) 2025, Sean McAvoy (@smcavoy) <seanmcavoy@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = """
---
module: lxd_storage_volume_info
short_description: Retrieve information about LXD storage volumes
version_added: 12.1.0
description:
  - Retrieve information about LXD storage volumes in a specific storage pool.
  - This module returns details about all volumes or a specific volume in a pool.
author: "Sean McAvoy (@smcavoy)"
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  pool:
    description:
      - Name of the storage pool to query for volumes.
      - This parameter is required.
    required: true
    type: str
  name:
    description:
      - Name of a specific storage volume to retrieve information about.
      - If not specified, information about all volumes in the pool are returned.
    type: str
  type:
    description:
      - Filter volumes by type.
      - Common types include V(container), V(virtual-machine), V(image), and V(custom).
      - If not specified, all volume types are returned.
    type: str
  project:
    description:
      - 'Project of the storage volume.
        See U(https://documentation.ubuntu.com/lxd/en/latest/projects/).'
    type: str
  url:
    description:
      - The Unix domain socket path or the https URL for the LXD server.
    default: unix:/var/lib/lxd/unix.socket
    type: str
  snap_url:
    description:
      - The Unix domain socket path when LXD is installed by snap package manager.
    default: unix:/var/snap/lxd/common/lxd/unix.socket
    type: str
  client_key:
    description:
      - The client certificate key file path.
      - If not specified, it defaults to C($HOME/.config/lxc/client.key).
    aliases: [key_file]
    type: path
  client_cert:
    description:
      - The client certificate file path.
      - If not specified, it defaults to C($HOME/.config/lxc/client.crt).
    aliases: [cert_file]
    type: path
  trust_password:
    description:
      - The client trusted password.
      - 'You need to set this password on the LXD server before
        running this module using the following command:
        C(lxc config set core.trust_password <some random password>)
        See U(https://www.stgraber.org/2016/04/18/lxd-api-direct-interaction/).'
      - If O(trust_password) is set, this module sends a request for
        authentication before sending any requests.
    type: str
"""

EXAMPLES = """
- name: Get information about all volumes in the default storage pool
  community.general.lxd_storage_volume_info:
    pool: default
  register: result

- name: Get information about a specific volume
  community.general.lxd_storage_volume_info:
    pool: default
    name: my-volume
  register: result

- name: Get information about all custom volumes in a pool
  community.general.lxd_storage_volume_info:
    pool: default
    type: custom
  register: result

- name: Get volume information via HTTPS connection
  community.general.lxd_storage_volume_info:
    url: https://127.0.0.1:8443
    trust_password: mypassword
    pool: default
    name: my-volume
  register: result

- name: Get volume information for a specific project
  community.general.lxd_storage_volume_info:
    project: myproject
    pool: default
  register: result

- name: Get container volumes only
  community.general.lxd_storage_volume_info:
    pool: default
    type: container
  register: result
"""

RETURN = """
storage_volumes:
  description: List of LXD storage volumes.
  returned: success
  type: list
  elements: dict
  sample: [
    {
      "name": "my-volume",
      "type": "custom",
      "used_by": [],
      "config": {
        "size": "10GiB"
      },
      "description": "My custom volume",
      "content_type": "filesystem",
      "location": "none"
    }
  ]
  contains:
    name:
      description: The name of the storage volume.
      type: str
      returned: success
    type:
      description: The type of the storage volume.
      type: str
      returned: success
    used_by:
      description: List of resources using this storage volume.
      type: list
      elements: str
      returned: success
    config:
      description: Configuration of the storage volume.
      type: dict
      returned: success
    description:
      description: Description of the storage volume.
      type: str
      returned: success
    content_type:
      description: Content type of the volume (filesystem or block).
      type: str
      returned: success
    location:
      description: Cluster member location for the volume.
      type: str
      returned: success
logs:
  description: The logs of requests and responses.
  returned: when ansible-playbook is invoked with -vvvv.
  type: list
  elements: dict
  sample: [
    {
      "type": "sent request",
      "request": {
        "method": "GET",
        "url": "/1.0/storage-pools/default/volumes",
        "json": null,
        "timeout": null
      },
      "response": {
        "json": {"type": "sync", "status": "Success"}
      }
    }
  ]
"""

import os
import typing as t
from urllib.parse import quote, urlencode

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lxd import (
    LXDClient,
    LXDClientException,
    default_cert_file,
    default_key_file,
)

# ANSIBLE_LXD_DEFAULT_URL is a default value of the lxd endpoint
ANSIBLE_LXD_DEFAULT_URL = "unix:/var/lib/lxd/unix.socket"
ANSIBLE_LXD_DEFAULT_SNAP_URL = "unix:/var/snap/lxd/common/lxd/unix.socket"

# API endpoints
LXD_API_VERSION = "1.0"
LXD_API_STORAGE_POOLS_ENDPOINT = f"/{LXD_API_VERSION}/storage-pools"


class LXDStorageVolumeInfo:
    def __init__(self, module: AnsibleModule) -> None:
        """Gather information about LXD storage volumes.

        :param module: Processed Ansible Module.
        :type module: AnsibleModule
        """
        self.module = module
        self.pool = self.module.params["pool"]
        self.name = self.module.params["name"]
        self.volume_type = self.module.params["type"]
        self.project = self.module.params["project"]

        self.key_file = self.module.params["client_key"]
        if self.key_file is None:
            self.key_file = default_key_file()
        self.cert_file = self.module.params["client_cert"]
        if self.cert_file is None:
            self.cert_file = default_cert_file()
        self.debug = self.module._verbosity >= 4

        # check if domain socket to be used
        snap_socket_path = self.module.params["snap_url"]
        if snap_socket_path.startswith("unix:"):
            snap_socket_path = snap_socket_path[5:]

        if self.module.params["url"] != ANSIBLE_LXD_DEFAULT_URL:
            self.url = self.module.params["url"]
        elif os.path.exists(snap_socket_path):
            self.url = self.module.params["snap_url"]
        else:
            self.url = self.module.params["url"]

        try:
            self.client = LXDClient(
                self.url,
                key_file=self.key_file,
                cert_file=self.cert_file,
                debug=self.debug,
            )
        except LXDClientException as e:
            self._fail_from_lxd_exception(e)

        self.trust_password = self.module.params["trust_password"]

    def _fail_from_lxd_exception(self, exception: LXDClientException) -> t.NoReturn:
        """Build failure parameters from LXDClientException and fail.

        :param exception: The LXDClientException instance
        :type exception: LXDClientException
        """
        fail_params = {}
        if self.client.debug and "logs" in exception.kwargs:
            fail_params["logs"] = exception.kwargs["logs"]
        self.module.fail_json(msg=exception.msg, changed=False, **fail_params)

    def _build_url(self, endpoint: str) -> str:
        """Build URL with project parameter if specified."""
        if self.project:
            return f"{endpoint}?{urlencode({'project': self.project})}"
        return endpoint

    def _check_pool_exists(self) -> None:
        """Verify that the storage pool exists."""
        url = self._build_url(f"{LXD_API_STORAGE_POOLS_ENDPOINT}/{quote(self.pool, safe='')}")
        resp_json = self.client.do("GET", url, ok_error_codes=[404])

        if resp_json["type"] == "error":
            if resp_json.get("error_code") == 404:
                self.module.fail_json(msg=f'Storage pool "{self.pool}" not found')
            else:
                self.module.fail_json(
                    msg=f'Failed to retrieve storage pool "{self.pool}": {resp_json.get("error", "Unknown error")}'
                )

    def _get_volume_list(self, recursion: int = 0) -> list:
        """Get list of all volumes in the storage pool.

        :param recursion: API recursion level (0 for URLs only, 1 for full objects)
        :type recursion: int
        :return: List of volume URLs (recursion=0) or volume objects (recursion=1)
        :rtype: list
        """
        endpoint = f"{LXD_API_STORAGE_POOLS_ENDPOINT}/{quote(self.pool, safe='')}/volumes"
        if recursion > 0:
            endpoint = f"{endpoint}?recursion={recursion}"
        url = self._build_url(endpoint)
        resp_json = self.client.do("GET", url, ok_error_codes=[])

        if resp_json["type"] == "error":
            self.module.fail_json(
                msg=f'Failed to retrieve volumes from pool "{self.pool}": {resp_json.get("error", "Unknown error")}',
                error_code=resp_json.get("error_code"),
            )

        # With recursion=0: list of volume URLs like ['/1.0/storage-pools/default/volumes/custom/my-volume']
        # With recursion=1: list of volume objects with full metadata
        return resp_json.get("metadata", [])

    def _get_volume_info(self, volume_type: str, volume_name: str) -> dict:
        """Get detailed information about a specific storage volume."""
        url = self._build_url(
            f"{LXD_API_STORAGE_POOLS_ENDPOINT}/{quote(self.pool, safe='')}/volumes/{quote(volume_type, safe='')}/{quote(volume_name, safe='')}"
        )
        resp_json = self.client.do("GET", url, ok_error_codes=[404])

        if resp_json["type"] == "error":
            if resp_json.get("error_code") == 404:
                self.module.fail_json(
                    msg=f'Storage volume "{volume_name}" of type "{volume_type}" not found in pool "{self.pool}"'
                )
            else:
                self.module.fail_json(
                    msg=f'Failed to retrieve volume "{volume_name}" of type "{volume_type}": {resp_json.get("error", "Unknown error")}'
                )

        return resp_json.get("metadata", {})

    def get_storage_volumes(self) -> list[dict]:
        """Retrieve storage volume information based on module parameters."""
        # First check if the pool exists
        self._check_pool_exists()

        storage_volumes = []

        if self.name:
            # Get information about a specific volume
            # We need to determine the type if not specified
            if self.volume_type:
                volume_info = self._get_volume_info(self.volume_type, self.name)
                storage_volumes.append(volume_info)
            else:
                # Try to find the volume by name using recursion for efficiency
                # This gets all volume objects in a single API call
                volumes = self._get_volume_list(recursion=1)
                found = False
                for volume in volumes:
                    if volume.get("name") == self.name:
                        storage_volumes.append(volume)
                        found = True
                        break

                if not found:
                    self.module.fail_json(msg=f'Storage volume "{self.name}" not found in pool "{self.pool}"')
        else:
            # Get information about all volumes in the pool using recursion
            # This retrieves all volume details in a single API call instead of one call per volume
            volumes = self._get_volume_list(recursion=1)
            for volume in volumes:
                # Apply type filter if specified
                if self.volume_type and volume.get("type") != self.volume_type:
                    continue
                storage_volumes.append(volume)

        return storage_volumes

    def run(self) -> None:
        """Run the main method."""
        try:
            if self.trust_password is not None:
                self.client.authenticate(self.trust_password)

            storage_volumes = self.get_storage_volumes()

            result_json = {"storage_volumes": storage_volumes}
            if self.client.debug:
                result_json["logs"] = self.client.logs
            self.module.exit_json(**result_json)
        except LXDClientException as e:
            self._fail_from_lxd_exception(e)


def main() -> None:
    """Ansible Main module."""
    module = AnsibleModule(
        argument_spec=dict(
            pool=dict(type="str", required=True),
            name=dict(type="str"),
            type=dict(type="str"),
            project=dict(type="str"),
            url=dict(type="str", default=ANSIBLE_LXD_DEFAULT_URL),
            snap_url=dict(type="str", default=ANSIBLE_LXD_DEFAULT_SNAP_URL),
            client_key=dict(type="path", aliases=["key_file"]),
            client_cert=dict(type="path", aliases=["cert_file"]),
            trust_password=dict(type="str", no_log=True),
        ),
        supports_check_mode=True,
    )

    lxd_info = LXDStorageVolumeInfo(module=module)
    lxd_info.run()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Dev Sinha (@DevSinha13) <devsinh@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_vmm_enhanced_lag_policy
version_added: "2.12.0"
short_description: Manage Enhanced LACP Policy for Virtual Machine Manager (VMM) in Cisco ACI (lacp:EnhancedLagPol)
description:
- Manage Enhanced LACP Policy for VMM domains on Cisco ACI fabrics.
- The Enhanced LACP Policy allows you to configure advanced Link Aggregation Control Protocol settings for virtual switches in VMM domains.

options:
  name:
    description:
    - The name of the Enhanced LACP Policy.
    type: str
  domain:
    description:
    - The name of the virtual domain profile where the Enhanced LACP Policy is applied.
    type: str
    aliases: [ domain_name, domain_profile ]
  vm_provider:
    description:
    - The virtualization platform provider for the VMM domain.
    type: str
    choices: [ cloudfoundry, kubernetes, microsoft, openshift, openstack, redhat, vmware ]
  lacp_mode:
    description:
    - The LACP mode for the policy.
    - Determines whether the policy initiates or responds to LACP negotiations.
    - The APIC defaults to C(active) when unset during creation.
    type: str
    choices: [ active, passive ]
  load_balancing_mode:
    description:
    - The load balancing algorithm for distributing traffic across links in the port channel.
    - See the APIC Management Information Model reference for more details.
    - The APIC defaults to C(src-dst-ip) when unset during creation
    type: str
    choices:
    - dst-ip
    - dst-ip-l4port
    - dst-ip-vlan
    - dst-ip-l4port-vlan
    - dst-mac
    - dst-l4port
    - src-ip
    - src-ip-l4port
    - src-ip-vlan
    - src-ip-l4port-vlan
    - src-mac
    - src-l4port
    - src-dst-ip
    - src-dst-ip-l4port
    - src-dst-ip-vlan
    - src-dst-ip-l4port-vlan
    - src-dst-mac
    - src-dst-l4port
    - src-port-id
    - vlan
  number_uplinks:
    description:
    - The minimum number of uplinks required for the port channel.
    - Must be a value between 2 and 8.
    - The APIC defaults to 2 when unset during creation
    type: int
  state:
    description:
    - The desired state of the Enhanced LACP Policy.
    - Use C(present) to create or update the policy.
    - Use C(absent) to delete the policy.
    - Use C(query) to retrieve information about the policy.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(vmm_domain) and I(vSwitch_policy) must exist before using this module in a playbook.
- The modules M(cisco.aci.aci_domain) and M(cisco.aci.aci_vmm_vswitch_policy) can be used for this.
seealso:
- module: cisco.aci.aci_domain
- module: cisco.aci.aci_vmm_vswitch_policy
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(lacp:EnhancedLagPol).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dev Sinha (@DevSinha13)
"""

EXAMPLES = r"""
- name: Create an Enhanced LACP Policy
  cisco.aci.aci_vmm_enhanced_lag_policy:
    host: apic.example.com
    username: admin
    password: SomeSecretPassword
    name: my_enhanced_lag_policy
    domain: my_vmm_domain
    vm_provider: vmware
    lacp_mode: active
    load_balancing_mode: src-dst-ip
    number_uplinks: 4
    state: present

- name: Update an existing Enhanced LACP Policy
  cisco.aci.aci_vmm_enhanced_lag_policy:
    host: apic.example.com
    username: admin
    password: SomeSecretPassword
    name: my_enhanced_lag_policy
    domain: my_vmm_domain
    vm_provider: vmware
    lacp_mode: passive
    load_balancing_mode: src-dst-ip-l4port
    number_uplinks: 6
    state: present

- name: Query a specific Enhanced LACP Policy
  cisco.aci.aci_vmm_enhanced_lag_policy:
    host: apic.example.com
    username: admin
    password: SomeSecretPassword
    name: my_enhanced_lag_policy
    domain: my_vmm_domain
    vm_provider: vmware
    state: query
  register: query_result

- name: Query all Enhanced LACP Policies in a VMM domain
  cisco.aci.aci_vmm_enhanced_lag_policy:
    host: apic.example.com
    username: admin
    password: SomeSecretPassword
    domain: my_vmm_domain
    vm_provider: vmware
    state: query
  register: query_all_result

- name: Delete an Enhanced LACP Policy
  cisco.aci.aci_vmm_enhanced_lag_policy:
    host: apic.example.com
    username: admin
    password: SomeSecretPassword
    name: my_enhanced_lag_policy
    domain: my_vmm_domain
    vm_provider: vmware
    state: absent
"""
RETURN = r"""
current:
  description: The existing configuration of the Enhanced LACP Policy from the APIC after the module has finished.
  returned: success
  type: list
  sample:
    [
        {
            "lacpEnhancedLagPol": {
                "attributes": {
                    "name": "test_enhanced_lag_policy",
                    "mode": "active",
                    "lbmode": "src-dst-ip",
                    "numLinks": "4",
                    "dn": "uni/vmmp-VMware/dom-test_vmm_dom/vswitchpolcont/enlacplagp-test_enhanced_lag_policy"
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC.
  returned: failure
  type: dict
  sample:
    {
        "code": "801",
        "text": "property name of enlacplagp-test_enhanced_lag_policy failed validation"
    }
proposed:
  description: The configuration sent to the APIC.
  returned: info
  type: dict
  sample:
    {
        "lacpEnhancedLagPol": {
            "attributes": {
                "name": "test_enhanced_lag_policy",
                "mode": "active",
                "lbmode": "src-dst-ip",
                "numLinks": "4"
            }
        }
    }
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import (
    ACIModule,
    aci_argument_spec,
    enhanced_lag_spec,
    aci_annotation_spec,
    aci_owner_spec,
)
from ansible_collections.cisco.aci.plugins.module_utils.constants import (
    VM_PROVIDER_MAPPING,
)


def main():

    # Remove nutanix from VM_PROVIDER_MAPPING as it is not supported
    CLEAN_VM_PROVIDER_MAPPING = VM_PROVIDER_MAPPING.copy()
    CLEAN_VM_PROVIDER_MAPPING.pop("nutanix")

    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(enhanced_lag_spec(name_is_required=False))
    argument_spec.update(
        domain=dict(type="str", aliases=["domain_name", "domain_profile"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        vm_provider=dict(type="str", choices=list(CLEAN_VM_PROVIDER_MAPPING)),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "domain", "vm_provider"]],
            ["state", "present", ["name", "domain", "vm_provider"]],
        ],
    )

    name = module.params.get("name")
    lacp_mode = module.params.get("lacp_mode")
    load_balancing_mode = module.params.get("load_balancing_mode")
    number_uplinks = module.params.get("number_uplinks")
    domain = module.params.get("domain")
    state = module.params.get("state")
    vm_provider = module.params.get("vm_provider")

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="vmmProvP",
            aci_rn="vmmp-{0}".format(CLEAN_VM_PROVIDER_MAPPING.get(vm_provider)),
            module_object=vm_provider,
            target_filter={"vendor": vm_provider},
        ),
        subclass_1=dict(
            aci_class="vmmDomP",
            aci_rn="dom-{0}".format(domain),
            module_object=domain,
            target_filter={"name": domain},
        ),
        subclass_2=dict(
            aci_class="vmmVSwitchPolicyCont",
            aci_rn="vswitchpolcont",
            module_object="vswitchpolcont",
        ),
        subclass_3=dict(
            aci_class="lacpEnhancedLagPol",
            aci_rn="enlacplagp-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="lacpEnhancedLagPol",
            class_config=dict(
                name=name,
                mode=lacp_mode,
                lbmode=load_balancing_mode,
                numLinks=number_uplinks,
            ),
        )

        aci.get_diff(aci_class="lacpEnhancedLagPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2012, Dag Wieers <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: mail
type: notification
short_description: Sends failure events through email
description:
  - This callback reports failures through email.
author:
  - Dag Wieers (@dagwieers)
requirements:
  - whitelisting in configuration
options:
  mta:
    description:
      - Mail Transfer Agent, server that accepts SMTP.
    type: str
    env:
      - name: SMTPHOST
    ini:
      - section: callback_mail
        key: smtphost
    default: localhost
  mtaport:
    description:
      - Mail Transfer Agent Port.
      - Port at which server SMTP.
    type: int
    ini:
      - section: callback_mail
        key: smtpport
    default: 25
  to:
    description:
      - Mail recipient.
    type: list
    elements: str
    ini:
      - section: callback_mail
        key: to
    default: [root]
  sender:
    description:
      - Mail sender.
      - This is required since community.general 6.0.0.
    type: str
    required: true
    ini:
      - section: callback_mail
        key: sender
  cc:
    description:
      - CC'd recipients.
    type: list
    elements: str
    ini:
      - section: callback_mail
        key: cc
  bcc:
    description:
      - BCC'd recipients.
    type: list
    elements: str
    ini:
      - section: callback_mail
        key: bcc
  message_id_domain:
    description:
      - The domain name to use for the L(Message-ID header, https://en.wikipedia.org/wiki/Message-ID).
      - The default is the hostname of the control node.
    type: str
    ini:
      - section: callback_mail
        key: message_id_domain
    version_added: 8.2.0
"""

import email.utils
import json
import os
import re
import smtplib

from ansible.module_utils.common.text.converters import to_bytes
from ansible.parsing.ajson import AnsibleJSONEncoder
from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    """This Ansible callback plugin mails errors to interested parties."""

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "notification"
    CALLBACK_NAME = "community.general.mail"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self, display=None):
        super().__init__(display=display)
        self.sender = None
        self.to = "root"
        self.smtphost = os.getenv("SMTPHOST", "localhost")
        self.smtpport = 25
        self.cc = None
        self.bcc = None

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super().set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.sender = self.get_option("sender")
        self.to = self.get_option("to")
        self.smtphost = self.get_option("mta")
        self.smtpport = self.get_option("mtaport")
        self.cc = self.get_option("cc")
        self.bcc = self.get_option("bcc")

    def mail(self, subject="Ansible error mail", body=None):
        if body is None:
            body = subject

        smtp = smtplib.SMTP(self.smtphost, port=self.smtpport)

        sender_address = email.utils.parseaddr(self.sender)
        if self.to:
            to_addresses = email.utils.getaddresses(self.to)
        if self.cc:
            cc_addresses = email.utils.getaddresses(self.cc)
        if self.bcc:
            bcc_addresses = email.utils.getaddresses(self.bcc)

        content = f"Date: {email.utils.formatdate()}\n"
        content += f"From: {email.utils.formataddr(sender_address)}\n"
        if self.to:
            content += f"To: {', '.join([email.utils.formataddr(pair) for pair in to_addresses])}\n"
        if self.cc:
            content += f"Cc: {', '.join([email.utils.formataddr(pair) for pair in cc_addresses])}\n"
        content += f"Message-ID: {email.utils.make_msgid(domain=self.get_option('message_id_domain'))}\n"
        content += f"Subject: {subject.strip()}\n\n"
        content += body

        addresses = to_addresses
        if self.cc:
            addresses += cc_addresses
        if self.bcc:
            addresses += bcc_addresses

        if not addresses:
            self._display.warning("No receiver has been specified for the mail callback plugin.")

        smtp.sendmail(self.sender, [address for name, address in addresses], to_bytes(content))

        smtp.quit()

    def subject_msg(self, multiline, failtype, linenr):
        msg = multiline.strip("\r\n").splitlines()[linenr]
        return f"{failtype}: {msg}"

    def indent(self, multiline, indent=8):
        return re.sub("^", " " * indent, multiline, flags=re.MULTILINE)

    def body_blob(self, multiline, texttype):
        """Turn some text output in a well-indented block for sending in a mail body"""
        intro = f"with the following {texttype}:\n\n"
        blob = "\n".join(multiline.strip("\r\n").splitlines())
        return f"{intro}{self.indent(blob)}\n"

    def mail_result(self, result, failtype):
        host = result._host.get_name()
        if not self.sender:
            self.sender = f'"Ansible: {host}" <root>'

        # Add subject
        if self.itembody:
            subject = self.itemsubject
        elif result._result.get("failed_when_result") is True:
            subject = "Failed due to 'failed_when' condition"
        elif result._result.get("msg"):
            subject = self.subject_msg(result._result["msg"], failtype, 0)
        elif result._result.get("stderr"):
            subject = self.subject_msg(result._result["stderr"], failtype, -1)
        elif result._result.get("stdout"):
            subject = self.subject_msg(result._result["stdout"], failtype, -1)
        elif result._result.get("exception"):  # Unrelated exceptions are added to output :-/
            subject = self.subject_msg(result._result["exception"], failtype, -1)
        else:
            subject = f"{failtype}: {result._task.name or result._task.action}"

        # Make playbook name visible (e.g. in Outlook/Gmail condensed view)
        body = f"Playbook: {os.path.basename(self.playbook._file_name)}\n"
        if result._task.name:
            body += f"Task: {result._task.name}\n"
        body += f"Module: {result._task.action}\n"
        body += f"Host: {host}\n"
        body += "\n"

        # Add task information (as much as possible)
        body += "The following task failed:\n\n"
        if "invocation" in result._result:
            body += self.indent(
                f"{result._task.action}: {json.dumps(result._result['invocation']['module_args'], indent=4)}\n"
            )
        elif result._task.name:
            body += self.indent(f"{result._task.name} ({result._task.action})\n")
        else:
            body += self.indent(f"{result._task.action}\n")
        body += "\n"

        # Add item / message
        if self.itembody:
            body += self.itembody
        elif result._result.get("failed_when_result") is True:
            fail_cond_list = "\n- ".join(result._task.failed_when)
            fail_cond = self.indent(f"failed_when:\n- {fail_cond_list}")
            body += f"due to the following condition:\n\n{fail_cond}\n\n"
        elif result._result.get("msg"):
            body += self.body_blob(result._result["msg"], "message")

        # Add stdout / stderr / exception / warnings / deprecations
        if result._result.get("stdout"):
            body += self.body_blob(result._result["stdout"], "standard output")
        if result._result.get("stderr"):
            body += self.body_blob(result._result["stderr"], "error output")
        if result._result.get("exception"):  # Unrelated exceptions are added to output :-/
            body += self.body_blob(result._result["exception"], "exception")
        if result._result.get("warnings"):
            for i in range(len(result._result.get("warnings"))):
                body += self.body_blob(result._result["warnings"][i], f"exception {i + 1}")
        if result._result.get("deprecations"):
            for i in range(len(result._result.get("deprecations"))):
                body += self.body_blob(result._result["deprecations"][i], f"exception {i + 1}")

        body += "and a complete dump of the error:\n\n"
        body += self.indent(f"{failtype}: {json.dumps(result._result, cls=AnsibleJSONEncoder, indent=4)}")

        self.mail(subject=subject, body=body)

    def v2_playbook_on_start(self, playbook):
        self.playbook = playbook
        self.itembody = ""

    def v2_runner_on_failed(self, result, ignore_errors=False):
        if ignore_errors:
            return

        self.mail_result(result, "Failed")

    def v2_runner_on_unreachable(self, result):
        self.mail_result(result, "Unreachable")

    def v2_runner_on_async_failed(self, result):
        self.mail_result(result, "Async failure")

    def v2_runner_item_on_failed(self, result):
        # Pass item information to task failure
        self.itemsubject = result._result["msg"]
        self.itembody += self.body_blob(
            json.dumps(result._result, cls=AnsibleJSONEncoder, indent=4), f"failed item dump '{result._result['item']}'"
        )



#!/usr/bin/python

# Copyright (c) 2012, Franck Cuny <franck@lumberjaph.net>
# Copyright (c) 2021, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: cpanm
short_description: Manages Perl library dependencies
description:
  - Manage Perl library dependencies using cpanminus.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - The Perl library to install. Valid values change according to the O(mode), see notes for more details.
      - Note that for installing from a local path the parameter O(from_path) should be used.
    aliases: [pkg]
  from_path:
    type: path
    description:
      - The local directory or C(tar.gz) file to install from.
  notest:
    description:
      - Do not run unit tests.
    type: bool
    default: false
  locallib:
    description:
      - Specify the install base to install modules.
    type: path
  mirror:
    description:
      - Specifies the base URL for the CPAN mirror to use.
    type: str
  mirror_only:
    description:
      - Use the mirror's index file instead of the CPAN Meta DB.
    type: bool
    default: false
  installdeps:
    description:
      - Only install dependencies.
    type: bool
    default: false
  install_recommendations:
    description:
      - If V(true), installs dependencies declared as recommends per META spec.
      - If V(false), it ensures the dependencies declared as recommends are not installed, overriding any decision made earlier
        in E(PERL_CPANM_OPT).
      - If parameter is not set, C(cpanm) uses its existing defaults.
      - When these dependencies fail to install, cpanm continues the installation, since they are just recommendation.
    type: bool
    version_added: 10.3.0
  install_suggestions:
    description:
      - If V(true), installs dependencies declared as suggests per META spec.
      - If V(false), it ensures the dependencies declared as suggests are not installed, overriding any decision made earlier
        in E(PERL_CPANM_OPT).
      - If parameter is not set, C(cpanm) uses its existing defaults.
      - When these dependencies fail to install, C(cpanm) continues the installation, since they are just suggestions.
    type: bool
    version_added: 10.3.0
  version:
    description:
      - Version specification for the perl module. When O(mode=new), C(cpanm) version operators are accepted.
    type: str
  executable:
    description:
      - Override the path to the C(cpanm) executable.
    type: path
  mode:
    description:
      - Controls the module behavior. See notes below for more details.
      - The default changed from V(compatibility) to V(new) in community.general 9.0.0.
        V(compatibility) was removed from community.general 13.0.0.
      - 'O(mode=new): The O(name) parameter may refer to a module name, a distribution file, a HTTP URL or a git repository
        URL as described in C(cpanminus) documentation. C(cpanm) version specifiers are recognized. This is the default mode
        from community.general 9.0.0 onwards.'
    type: str
    choices: [new]
    default: new
    version_added: 3.0.0
  name_check:
    description:
      - When O(mode=new), this parameter can be used to check if there is a module O(name) installed (at O(version), when
        specified).
    type: str
    version_added: 3.0.0
notes:
  - Please note that U(http://search.cpan.org/dist/App-cpanminus/bin/cpanm, cpanm) must be installed on the remote host.
seealso:
  - name: C(cpanm) command manual page
    description: Manual page for the command.
    link: https://metacpan.org/dist/App-cpanminus/view/bin/cpanm
author:
  - "Franck Cuny (@fcuny)"
  - "Alexei Znamensky (@russoz)"
"""

EXAMPLES = r"""
- name: Install Dancer perl package
  community.general.cpanm:
    name: Dancer

- name: Install version 0.99_05 of the Plack perl package
  community.general.cpanm:
    name: MIYAGAWA/Plack-0.99_05.tar.gz

- name: Install Dancer into the specified locallib
  community.general.cpanm:
    name: Dancer
    locallib: /srv/webapps/my_app/extlib

- name: Install perl dependencies from local directory
  community.general.cpanm:
    from_path: /srv/webapps/my_app/src/

- name: Install Dancer perl package without running the unit tests in indicated locallib
  community.general.cpanm:
    name: Dancer
    notest: true
    locallib: /srv/webapps/my_app/extlib

- name: Install Dancer perl package from a specific mirror
  community.general.cpanm:
    name: Dancer
    mirror: 'http://cpan.cpantesters.org/'

- name: Install Dancer perl package into the system root path
  become: true
  community.general.cpanm:
    name: Dancer

- name: Install Dancer if it is not already installed OR the installed version is older than version 1.0
  community.general.cpanm:
    name: Dancer
    version: '1.0'
"""

RETURN = r"""
cpanm_version:
  description: Version of CPANMinus.
  type: str
  returned: always
  sample: "1.7047"
  version_added: 10.0.0
"""


import os
import re

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt
from ansible_collections.community.general.plugins.module_utils._module_helper import ModuleHelper


class CPANMinus(ModuleHelper):
    output_params = ["name", "version"]
    module = dict(
        argument_spec=dict(
            name=dict(type="str", aliases=["pkg"]),
            version=dict(type="str"),
            from_path=dict(type="path"),
            notest=dict(type="bool", default=False),
            locallib=dict(type="path"),
            mirror=dict(type="str"),
            mirror_only=dict(type="bool", default=False),
            installdeps=dict(type="bool", default=False),
            install_recommendations=dict(type="bool"),
            install_suggestions=dict(type="bool"),
            executable=dict(type="path"),
            mode=dict(type="str", default="new", choices=["new"]),
            name_check=dict(type="str"),
        ),
        required_one_of=[("name", "from_path")],
    )
    command = "cpanm"
    command_args_formats = dict(
        notest=cmd_runner_fmt.as_bool("--notest"),
        locallib=cmd_runner_fmt.as_opt_val("--local-lib"),
        mirror=cmd_runner_fmt.as_opt_val("--mirror"),
        mirror_only=cmd_runner_fmt.as_bool("--mirror-only"),
        installdeps=cmd_runner_fmt.as_bool("--installdeps"),
        install_recommendations=cmd_runner_fmt.as_bool("--with-recommends", "--without-recommends", ignore_none=True),
        install_suggestions=cmd_runner_fmt.as_bool("--with-suggests", "--without-suggests", ignore_none=True),
        pkg_spec=cmd_runner_fmt.as_list(),
        cpanm_version=cmd_runner_fmt.as_fixed("--version"),
    )

    def __init_module__(self):
        v = self.vars
        if v.name and v.from_path:
            self.do_raise("Parameters 'name' and 'from_path' are mutually exclusive when 'mode=new'")

        self.command = v.executable if v.executable else self.command
        self.runner = CmdRunner(self.module, self.command, self.command_args_formats, check_rc=True)
        self.vars.binary = self.runner.binary

        with self.runner("cpanm_version") as ctx:
            rc, out, err = ctx.run()
            line = out.split("\n")[0]
            match = re.search(r"version\s+([\d\.]+)\s+", line)
            if not match:
                self.do_raise(f"Failed to determine version number. First line of output: {line}")
            self.vars.cpanm_version = match.group(1)

    def _is_package_installed(self, name, locallib, version):
        def process(rc, out, err):
            return rc == 0

        if name is None or name.endswith(".tar.gz"):
            return False
        version = "" if version is None else f" {version}"

        env = {"PERL5LIB": f"{locallib}/lib/perl5"} if locallib else {}
        runner = CmdRunner(
            self.module, ["perl", "-le"], {"mod": cmd_runner_fmt.as_list()}, check_rc=False, environ_update=env
        )
        with runner("mod", output_process=process) as ctx:
            return ctx.run(mod=f"use {name}{version};")

    def sanitize_pkg_spec_version(self, pkg_spec, version):
        if version is None:
            return pkg_spec
        if pkg_spec.endswith(".tar.gz"):
            self.do_raise(msg="parameter 'version' must not be used when installing from a file")
        if os.path.isdir(pkg_spec):
            self.do_raise(msg="parameter 'version' must not be used when installing from a directory")
        if pkg_spec.endswith(".git"):
            if version.startswith("~"):
                self.do_raise(msg="operator '~' not allowed in version parameter when installing from git repository")
            version = version if version.startswith("@") else f"@{version}"
        elif version[0] not in ("@", "~"):
            version = f"~{version}"
        return pkg_spec + version

    def __run__(self):
        def process(rc, out, err):
            return "is up to date" not in err and "is up to date" not in out

        v = self.vars
        pkg_param = "from_path" if v.from_path else "name"

        installed = self._is_package_installed(v.name_check, v.locallib, v.version) if v.name_check else False
        if installed:
            return
        pkg_spec = self.sanitize_pkg_spec_version(v[pkg_param], v.version)

        with self.runner(
            [
                "notest",
                "locallib",
                "mirror",
                "mirror_only",
                "installdeps",
                "install_recommendations",
                "install_suggestions",
                "pkg_spec",
            ],
            output_process=process,
        ) as ctx:
            self.changed = ctx.run(pkg_spec=pkg_spec)


def main():
    CPANMinus.execute()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class InterfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(InterfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""
                ^interface\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": 'interface {{ name }}',
            "result": {
                '{{ name }}': {
                    'name': '{{ name }}',
                },
            },
            "shared": True,
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                \s+description\s(?P<description>.+$)
                $""", re.VERBOSE,
            ),
            "setval": "description {{ description }}",
            "result": {
                '{{ name }}': {
                    'description': "'{{ description }}'",
                },
            },
        },
        {
            "name": "enabled",
            "getval": re.compile(
                r"""
                (?P<negate>\s+no)?
                (?P<shutdown>\s+shutdown)
                $""", re.VERBOSE,
            ),
            "setval": "shutdown",
            "result": {
                '{{ name }}': {
                    'enabled': "{{ False if shutdown is defined and negate is not defined else True }}",
                },
            },
        },
        {
            "name": "speed",
            "getval": re.compile(
                r"""
                \s+speed\s(?P<speed>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "speed {{ speed|string }}",
            "result": {
                '{{ name }}': {
                    'speed': "{{ speed }}",
                },
            },
        },
        {
            "name": "mtu",
            "getval": re.compile(
                r"""
                \s+mtu\s(?P<mtu>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "mtu {{ mtu|string }}",
            "result": {
                '{{ name }}': {
                    'mtu': "{{ mtu }}",
                },
            },
        },
        {
            "name": "duplex",
            "getval": re.compile(
                r"""
                \s+duplex\s(?P<duplex>full|half|auto)
                $""", re.VERBOSE,
            ),
            "setval": "duplex {{ duplex }}",
            "result": {
                '{{ name }}': {
                    'duplex': "{{ duplex }}",
                },
            },
        },
        {
            "name": "ip_forward",
            "getval": re.compile(
                r"""
                \s+ip\s(?P<ip_forward>forward)
                $""", re.VERBOSE,
            ),
            "setval": "ip forward",
            "result": {
                '{{ name }}': {
                    'ip_forward': "{{ True if ip_forward is defined }}",
                },
            },
        },
        {
            "name": "fabric_forwarding_anycast_gateway",
            "getval": re.compile(
                r"""
                \s+fabric\sforwarding
                \smode\sanycast-gateway
                $""", re.VERBOSE,
            ),
            "setval": "fabric forwarding mode anycast-gateway",
            "result": {
                '{{ name }}': {
                    'fabric_forwarding_anycast_gateway': "{{ True }}",
                },
            },
        },
        {  # only applicable for switches
            "name": "mode",
            "getval": re.compile(
                r"""
                (?P<negate>\s+no)?
                (?P<switchport>\s+switchport)
                $""", re.VERBOSE,
            ),
            "setval": "switchport",
            "result": {
                '{{ name }}': {
                    'mode': "{{ 'layer2' if switchport is defined and negate is not defined else 'layer3' }}",
                },
            },
        },
        {
            "name": "mac_address",
            "getval": re.compile(
                r"""
                \s+mac-address
                (\s(?P<mac_address>.+))
                $""", re.VERBOSE,
            ),
            "setval": "mac-address {{ mac_address }}",
            "result": {
                '{{ name }}': {
                    'mac_address': "{{ mac_address }}",
                },
            },
        },
        {
            "name": "logging.link_status",
            "getval": re.compile(
                r"""
                \s+logging\sevent
                \sport(\s(?P<link_status>link-status))
                $""", re.VERBOSE,
            ),
            "setval": "logging event port link-status",
            "result": {
                '{{ name }}': {
                    "logging": {
                        "link_status": "{{ True if link_status is defined }}",
                    },
                },
            },
        },
        {
            "name": "logging.trunk_status",
            "getval": re.compile(
                r"""
                \s+logging\sevent
                \sport(\s(?P<trunk_status>trunk-status))
                $""", re.VERBOSE,
            ),
            "setval": "logging event port trunk-status",
            "result": {
                '{{ name }}': {
                    "logging": {
                        "trunk_status": "{{ True if trunk_status is defined }}",
                    },
                },
            },
        },
        {
            "name": "snmp.trap.link_status",
            "getval": re.compile(
                r"""
                \s+snmp\strap(\s(?P<link_status>link-status))
                $""", re.VERBOSE,
            ),
            "setval": "snmp trap link-status",
            "result": {
                '{{ name }}': {
                    "snmp": {
                        "trap": {
                            "link_status": "{{ True if link_status is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "service_policy.input",
            "getval": re.compile(
                r"""
                \s+service-policy\sinput
                (\s(?P<service_policy_input>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "service-policy input {{ service_policy.input }}",
            "result": {
                '{{ name }}': {
                    "service_policy": {
                        "input": "{{ service_policy_input }}",
                    },
                },
            },
        },
        {
            "name": "service_policy.output",
            "getval": re.compile(
                r"""
                \s+service-policy\soutput
                (\s(?P<service_policy_output>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "service-policy output {{ service_policy.output }}",
            "result": {
                '{{ name }}': {
                    "service_policy": {
                        "output": "{{ service_policy_output }}",
                    },
                },
            },
        },
        {
            "name": "service_policy.type_options.qos.input",
            "getval": re.compile(
                r"""
                \s+service-policy\stype\sqos
                (\sinput\s(?P<service_policy_input>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "service-policy type qos input {{ service_policy.type_options.qos.input }}",
            "result": {
                '{{ name }}': {
                    "service_policy": {
                        "type_options": {
                            "qos": {
                                "input": "{{ service_policy_input }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "service_policy.type_options.qos.output",
            "getval": re.compile(
                r"""
                \s+service-policy\stype\sqos
                (\soutput\s(?P<service_policy_output>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "service-policy type qos output {{ service_policy.type_options.qos.output }}",
            "result": {
                '{{ name }}': {
                    "service_policy": {
                        "type_options": {
                            "qos": {
                                "output": "{{ service_policy_output }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "service_policy.type_options.queuing.input",
            "getval": re.compile(
                r"""
                \s+service-policy\stype\squeuing
                (\sinput\s(?P<service_policy_input>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "service-policy type queuing input {{ service_policy.type_options.queuing.input }}",
            "result": {
                '{{ name }}': {
                    "service_policy": {
                        "type_options": {
                            "queuing": {
                                "input": "{{ service_policy_input }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "service_policy.type_options.queuing.output",
            "getval": re.compile(
                r"""
                \s+service-policy\stype\squeuing
                (\soutput\s(?P<service_policy_output>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "service-policy type queuing output {{ service_policy.type_options.queuing.output }}",
            "result": {
                '{{ name }}': {
                    "service_policy": {
                        "type_options": {
                            "queuing": {
                                "output": "{{ service_policy_output }}",
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: reduce_on_network
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import (
    _address_normalizer,
    _need_netaddr,
    _range_checker,
    ipaddr,
)


__metaclass__ = type


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

DOCUMENTATION = """
    name: reduce_on_network
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: This filter reduces a list of addresses to only the addresses that match a given network.
    description:
    - This filter reduces a list of addresses to only the addresses that match a given network.
    - To check whether multiple addresses belong to a network, use the reduce_on_network filter.
    options:
        value:
            description: the list of addresses to filter on.
            type: list
            elements: str
            required: True
        network:
            description: The network to validate against.
            type: str
    notes:
"""

EXAMPLES = r"""

- name: To check whether multiple addresses belong to a network, use the reduce_on_network filter.
  debug:
    msg: "{{ ['192.168.0.34', '10.3.0.3', '192.168.2.34'] | ansible.utils.reduce_on_network( '192.168.0.0/24' ) }}"

# TASK [To check whether multiple addresses belong to a network, use the reduce_on_network filter.] ***********
# task path: /Users/amhatre/ansible-collections/playbooks/test_reduce_on_network.yaml:7
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": [
#         "192.168.0.34"
#     ]
# }
"""

RETURN = """
  data:
    type: bool
    description:
      - Returns the filtered list of addresses belonging to the network.

"""


@pass_environment
def _reduce_on_network(*args, **kwargs):
    """This filter returns whether an address or a network passed as argument is in a network."""
    keys = ["value", "network"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="reduce_on_network")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return reduce_on_network(**updated_data)


def reduce_on_network(value, network):
    """
    Reduces a list of addresses to only the addresses that match a given network.
    :param: value: The list of addresses to filter on.
    :param: network: The network to validate against.
    :return: The reduced list of addresses.
    """
    # normalize network variable into an ipaddr
    n = _address_normalizer(network)

    # get first and last addresses as integers to compare value and test; or cathes value when case is /32
    n_first = ipaddr(ipaddr(n, "network") or ipaddr(n, "address"), "int")
    n_last = ipaddr(ipaddr(n, "broadcast") or ipaddr(n, "address"), "int")

    # create an empty list to fill and return
    r = []

    for address in value:
        # normalize address variables into an ipaddr
        a = _address_normalizer(address)

        # get first and last addresses as integers to compare value and test; or cathes value when case is /32
        a_first = ipaddr(ipaddr(a, "network") or ipaddr(a, "address"), "int")
        a_last = ipaddr(ipaddr(a, "broadcast") or ipaddr(a, "address"), "int")

        if _range_checker(a_first, n_first, n_last) and _range_checker(a_last, n_first, n_last):
            r.append(address)

    return r


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "reduce_on_network": _reduce_on_network,
    }

    def filters(self):
        """ipaddr filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
#
# Copyright (c) 2023, Dominik Kukacka <dominik.kukacka@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: "ipbase_info"
version_added: "7.0.0"
short_description: "Retrieve IP geolocation and other facts of a host's IP address using the ipbase.com API"
description:
  - Retrieve IP geolocation and other facts of a host's IP address using the ipbase.com API.
author: "Dominik Kukacka (@dominikkukacka)"
extends_documentation_fragment:
  - "community.general._attributes"
  - "community.general._attributes.info_module"
options:
  ip:
    description:
      - The IP you want to get the info for. If not specified the API detects the IP automatically.
    type: str
  apikey:
    description:
      - The API key for the request if you need more requests.
    type: str
  hostname:
    description:
      - If the O(hostname) parameter is set to V(true), the API response contains the hostname of the IP.
    type: bool
    default: false
  language:
    description:
      - An ISO Alpha 2 Language Code for localizing the IP data.
    type: str
    default: "en"
notes:
  - Check U(https://ipbase.com/) for more information.
"""

EXAMPLES = r"""
- name: "Get IP geolocation information of the primary outgoing IP"
  community.general.ipbase_info:
  register: my_ip_info

- name: "Get IP geolocation information of a specific IP"
  community.general.ipbase_info:
    ip: "8.8.8.8"
  register: my_ip_info


- name: "Get IP geolocation information of a specific IP with all other possible parameters"
  community.general.ipbase_info:
    ip: "8.8.8.8"
    apikey: "xxxxxxxxxxxxxxxxxxxxxx"
    hostname: true
    language: "de"
  register: my_ip_info
"""

RETURN = r"""
data:
  description: "JSON parsed response from ipbase.com. Please refer to U(https://ipbase.com/docs/info) for the detailed structure
    of the response."
  returned: success
  type: dict
  sample:
    {
      "ip": "1.1.1.1",
      "hostname": "one.one.one.one",
      "type": "v4",
      "range_type": {
        "type": "PUBLIC",
        "description": "Public address"
      },
      "connection": {
        "asn": 13335,
        "organization": "Cloudflare, Inc.",
        "isp": "APNIC Research and Development",
        "range": "1.1.1.1/32"
      },
      "location": {
        "geonames_id": 5332870,
        "latitude": 34.053611755371094,
        "longitude": -118.24549865722656,
        "zip": "90012",
        "continent": {
          "code": "NA",
          "name": "North America",
          "name_translated": "North America"
        },
        "country": {
          "alpha2": "US",
          "alpha3": "USA",
          "calling_codes": [
            "+1"
          ],
          "currencies": [
            {
              "symbol": "$",
              "name": "US Dollar",
              "symbol_native": "$",
              "decimal_digits": 2,
              "rounding": 0,
              "code": "USD",
              "name_plural": "US dollars"
            }
          ],
          "emoji": "...",
          "ioc": "USA",
          "languages": [
            {
              "name": "English",
              "name_native": "English"
            }
          ],
          "name": "United States",
          "name_translated": "United States",
          "timezones": [
            "America/New_York",
            "America/Detroit",
            "America/Kentucky/Louisville",
            "America/Kentucky/Monticello",
            "America/Indiana/Indianapolis",
            "America/Indiana/Vincennes",
            "America/Indiana/Winamac",
            "America/Indiana/Marengo",
            "America/Indiana/Petersburg",
            "America/Indiana/Vevay",
            "America/Chicago",
            "America/Indiana/Tell_City",
            "America/Indiana/Knox",
            "America/Menominee",
            "America/North_Dakota/Center",
            "America/North_Dakota/New_Salem",
            "America/North_Dakota/Beulah",
            "America/Denver",
            "America/Boise",
            "America/Phoenix",
            "America/Los_Angeles",
            "America/Anchorage",
            "America/Juneau",
            "America/Sitka",
            "America/Metlakatla",
            "America/Yakutat",
            "America/Nome",
            "America/Adak",
            "Pacific/Honolulu"
          ],
          "is_in_european_union": false,
          "fips": "US",
          "geonames_id": 6252001,
          "hasc_id": "US",
          "wikidata_id": "Q30"
        },
        "city": {
          "fips": "644000",
          "alpha2": null,
          "geonames_id": 5368753,
          "hasc_id": null,
          "wikidata_id": "Q65",
          "name": "Los Angeles",
          "name_translated": "Los Angeles"
        },
        "region": {
          "fips": "US06",
          "alpha2": "US-CA",
          "geonames_id": 5332921,
          "hasc_id": "US.CA",
          "wikidata_id": "Q99",
          "name": "California",
          "name_translated": "California"
        }
      },
      "tlds": [
        ".us"
      ],
      "timezone": {
        "id": "America/Los_Angeles",
        "current_time": "2023-05-04T04:30:28-07:00",
        "code": "PDT",
        "is_daylight_saving": true,
        "gmt_offset": -25200
      },
      "security": {
        "is_anonymous": false,
        "is_datacenter": false,
        "is_vpn": false,
        "is_bot": false,
        "is_abuser": true,
        "is_known_attacker": true,
        "is_proxy": false,
        "is_spam": false,
        "is_tor": false,
        "is_icloud_relay": false,
        "threat_score": 100
      },
      "domains": {
        "count": 10943,
        "domains": [
          "eliwise.academy",
          "accountingprose.academy",
          "pistola.academy",
          "1and1-test-ntlds-fr.accountant",
          "omnergy.africa"
        ]
      }
    }
"""

from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

USER_AGENT = "ansible-community.general.ipbase_info/0.1.0"
BASE_URL = "https://api.ipbase.com/v2/info"


class IpbaseInfo:
    def __init__(self, module):
        self.module = module

    def _get_url_data(self, url):
        response, info = fetch_url(
            self.module,
            url,
            force=True,
            timeout=10,
            headers={
                "Accept": "application/json",
                "User-Agent": USER_AGENT,
            },
        )

        if info["status"] != 200:
            self.module.fail_json(msg=f"The API request to ipbase.com returned an error status code {info['status']}")
        else:
            try:
                content = response.read()
                result = self.module.from_json(content.decode("utf8"))
            except ValueError:
                self.module.fail_json(msg=f"Failed to parse the ipbase.com response: {url} {content}")
            else:
                return result

    def info(self):
        ip = self.module.params["ip"]
        apikey = self.module.params["apikey"]
        hostname = self.module.params["hostname"]
        language = self.module.params["language"]

        url = BASE_URL

        params = {}
        if ip:
            params["ip"] = ip

        if apikey:
            params["apikey"] = apikey

        if hostname:
            params["hostname"] = 1

        if language:
            params["language"] = language

        if params:
            url += f"?{urlencode(params)}"

        return self._get_url_data(url)


def main():
    module_args = dict(
        ip=dict(type="str", no_log=False),
        apikey=dict(type="str", no_log=True),
        hostname=dict(type="bool", no_log=False, default=False),
        language=dict(type="str", no_log=False, default="en"),
    )

    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True,
    )

    ipbase = IpbaseInfo(module)
    module.exit_json(**ipbase.info())


if __name__ == "__main__":
    main()



# (c) 2020, Alexei Znamensky <russoz@gmail.com>
# Copyright (c) 2020, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t


class StateMixin:
    state_param: str = "state"
    default_state: str | None = None

    def _state(self) -> str:
        state: str = self.module.params.get(self.state_param)  # type: ignore[attr-defined]
        return self.default_state if state is None else state

    def _method(self, state: str) -> str:
        return f"{self.state_param}_{state}"

    def __run__(self):
        state = self._state()
        self.vars.state = state

        # resolve aliases
        if state not in self.module.params:
            aliased = [name for name, param in self.module.argument_spec.items() if state in param.get("aliases", [])]
            if aliased:
                state = aliased[0]
                self.vars.effective_state = state

        method = self._method(state)
        if not hasattr(self, method):
            return self.__state_fallback__()
        func = getattr(self, method)
        return func()

    def __state_fallback__(self) -> t.NoReturn:
        raise ValueError(f"Cannot find method: {self._method(self._state())}")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_voip_domain_h323_gateway
short_description: Manages voip-domain-h323-gateway objects on Checkpoint over Web Services API
description:
  - Manages voip-domain-h323-gateway objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R82.10 management version.
version_added: "6.7.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  endpoints_domain:
    description:
      - The related endpoints domain to which the VoIP domain will connect. Identified by name or UID.
    type: str
  installed_at:
    description:
      - The machine the VoIP is installed at. Identified by name or UID.
    type: str
  routing_mode:
    description:
      - The routing mode of the VoIP Domain H323 gateway.
    type: dict
    suboptions:
      call_setup:
        description:
          - Indicates whether the routing mode includes call setup (Q.931).
        type: bool
      call_setup_and_call_control:
        description:
          - Indicates whether the routing mode includes both call setup (Q.931) and call control (H.245).
        type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-voip-domain-h323-gateway
  cp_mgmt_voip_domain_h323_gateway:
    endpoints_domain: new_group
    installed_at: test_host
    name: vdhgw1
    routing_mode:
      call_setup: true
      call_setup_and_call_control: false
    state: present

- name: set-voip-domain-h323-gateway
  cp_mgmt_voip_domain_h323_gateway:
    endpoints_domain: IPv6_Link_Local_Hosts
    name: vdhgw1
    state: present

- name: delete-voip-domain-h323-gateway
  cp_mgmt_voip_domain_h323_gateway:
    name: vdhgw1
    state: absent
"""

RETURN = """
cp_mgmt_voip_domain_h323_gateway:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_objects, api_call


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        endpoints_domain=dict(type='str'),
        installed_at=dict(type='str'),
        routing_mode=dict(type='dict', options=dict(
            call_setup=dict(type='bool'),
            call_setup_and_call_control=dict(type='bool')
        )),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        tags=dict(type='list', elements='str'),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    api_call_object = 'voip-domain-h323-gateway'

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wafv2_web_acl
version_added: 1.5.0
author:
  - "Markus Bergholz (@markuman)"
short_description: Create and delete WAF Web ACLs
description:
  - Create, modify or delete AWS WAF v2 web ACLs (not for classic WAF).
  - See docs at U(https://docs.aws.amazon.com/waf/latest/developerguide/waf-chapter.html)
options:
    state:
      description:
        - Whether the rule is present or absent.
      choices: ["present", "absent"]
      required: true
      type: str
    name:
      description:
        - The name of the web acl.
      required: true
      type: str
    scope:
      description:
        - Geographical scope of the web acl.
      required: true
      choices: ["CLOUDFRONT", "REGIONAL"]
      type: str
    description:
      description:
        - Description of wafv2 web acl.
      type: str
    default_action:
      description:
        - Default action of the wafv2 web acl.
      choices: ["Block", "Allow"]
      type: str
    sampled_requests:
      description:
        - Whether to store a sample of the web requests, true or false.
      type: bool
      default: false
    cloudwatch_metrics:
      description:
        - Enable cloudwatch metric for wafv2 web acl.
      type: bool
      default: true
    metric_name:
      description:
        - Name of cloudwatch metrics.
        - If not given and cloudwatch_metrics is enabled, the name of the web acl itself will be taken.
      type: str
    rules:
      description:
        - The Rule statements used to identify the web requests that you want to allow, block, or count.
        - For a list of managed rules see U(https://docs.aws.amazon.com/waf/latest/developerguide/aws-managed-rule-groups-list.html).
      type: list
      elements: dict
      suboptions:
        name:
          description:
            - The name of the wafv2 rule
          type: str
        priority:
          description:
            - The rule priority
          type: int
        action:
          description:
            - Wether a rule is blocked, allowed or counted.
          type: dict
        visibility_config:
          description:
            - Visibility of single wafv2 rule.
          type: dict
        statement:
          description:
            - Rule configuration.
          type: dict
    custom_response_bodies:
      description:
        - A map of custom response keys and content bodies. Define response bodies here and reference them in the rules by providing
        - the key of the body dictionary element.
        - Each element must have a unique dict key and in the dict two keys for I(content_type) and I(content).
      type: dict
      version_added: 3.1.0
    purge_rules:
      description:
        - When set to C(no), keep the existing load balancer rules in place. Will modify and add, but will not delete.
      default: true
      type: bool

notes:
  - Support for the I(purge_tags) parameter was added in release 4.0.0.

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create test web acl
  community.aws.wafv2_web_acl:
    name: test05
    description: hallo eins
    scope: REGIONAL
    default_action: Allow
    sampled_requests: false
    cloudwatch_metrics: true
    metric_name: test05-acl-metric
    rules:
      - name: zwei
        priority: 0
        action:
          block: {}
        visibility_config:
          sampled_requests_enabled: true
          cloud_watch_metrics_enabled: true
          metric_name: ddos
        statement:
          xss_match_statement:
            field_to_match:
              body: {}
            text_transformations:
              - type: NONE
                priority: 0
      - name: admin_protect
        priority: 1
        override_action:
          none: {}
        visibility_config:
          sampled_requests_enabled: true
          cloud_watch_metrics_enabled: true
          metric_name: fsd
        statement:
          managed_rule_group_statement:
            vendor_name: AWS
            name: AWSManagedRulesAdminProtectionRuleSet

      # AWS Managed Bad Input Rule Set
      # but allow PROPFIND_METHOD used e.g. by webdav
      - name: bad_input_protect_whitelist_webdav
        priority: 2
        override_action:
          none: {}
        visibility_config:
          sampled_requests_enabled: true
          cloud_watch_metrics_enabled: true
          metric_name: bad_input_protect
        statement:
          managed_rule_group_statement:
            vendor_name: AWS
            name: AWSManagedRulesKnownBadInputsRuleSet
            excluded_rules:
              - name: PROPFIND_METHOD

      # Rate Limit example. 1500 req/5min
      # counted for two domains via or_statement. login.mydomain.tld and api.mydomain.tld
      - name: rate_limit_example
        priority: 3
        action:
          block: {}
        visibility_config:
          sampled_requests_enabled: true
          cloud_watch_metrics_enabled: true
          metric_name: mydomain-ratelimit
        statement:
          rate_based_statement:
            limit: 1500
            aggregate_key_type: IP
            scope_down_statement:
              or_statement:
                statements:
                  - byte_match_statement:
                      search_string: login.mydomain.tld
                      positional_constraint: CONTAINS
                      field_to_match:
                        single_header:
                          name: host
                      text_transformations:
                        - type: LOWERCASE
                          priority: 0
                  - byte_match_dtatement:
                      search_string: api.mydomain.tld
                      positional_constraint: CONTAINS
                      field_to_match:
                        single_header:
                          name: host
                      text_transformations:
                        - type: LOWERCASE
                          priority: 0
    purge_rules: true
    tags:
      A: B
      C: D
    state: present

- name: Create IP filtering web ACL
  community.aws.wafv2_web_acl:
    name: ip-filtering-traffic
    description: ACL that filters web traffic based on rate limits and whitelists some IPs
    scope: REGIONAL
    default_action: Allow
    sampled_requests: true
    cloudwatch_metrics: true
    metric_name: ip-filtering-traffic
    rules:
      - name: whitelist-own-IPs
        priority: 0
        action:
          allow: {}
        statement:
          ip_set_reference_statement:
            arn: 'arn:aws:wafv2:us-east-1:123456789012:regional/ipset/own-public-ips/1c4bdfc4-0f77-3b23-5222-123123123'
        visibility_config:
          sampled_requests_enabled: true
          cloud_watch_metrics_enabled: true
          metric_name: waf-acl-rule-whitelist-own-IPs
      - name: rate-limit-per-IP
        priority: 1
        action:
          block:
            custom_response:
              response_code: 429
              custom_response_body_key: too_many_requests
        statement:
          rate_based_statement:
            limit: 5000
            aggregate_key_type: IP
        visibility_config:
          sampled_requests_enabled: true
          cloud_watch_metrics_enabled: true
          metric_name: waf-acl-rule-rate-limit-per-IP
        purge_rules: true
    custom_response_bodies:
      too_many_requests:
        content_type: APPLICATION_JSON
        content: '{ message: "Your request has been blocked due to too many HTTP requests coming from your IP" }'
    region: us-east-1
    state: present
"""

RETURN = r"""
arn:
  description: web acl arn
  sample: arn:aws:wafv2:eu-central-1:123456789012:regional/webacl/test05/318c1ab9-fa74-4b3b-a974-f92e25106f61
  type: str
  returned: Always, as long as the web acl exists
description:
  description: Description of the web acl
  sample: Some web acl description
  returned: Always, as long as the web acl exists
  type: str
capacity:
  description: Current capacity of the web acl
  sample: 140
  returned: Always, as long as the web acl exists
  type: int
name:
  description: Web acl name
  sample: test02
  returned: Always, as long as the web acl exists
  type: str
default_action:
  description: Default action of ACL
  returned: Always, as long as the web acl exists
  sample:
    allow: {}
  type: dict
rules:
  description: Current rules of the web acl
  returned: Always, as long as the web acl exists
  type: list
  sample:
    - name: admin_protect
      override_action:
        none: {}
      priority: 1
      statement:
        managed_rule_group_statement:
          name: AWSManagedRulesAdminProtectionRuleSet
          vendor_name: AWS
      visibility_config:
        cloud_watch_metrics_enabled: true
        metric_name: admin_protect
        sampled_requests_enabled: true
custom_response_bodies:
  description: Custom response body configurations to be used in rules
  type: dict
  sample:
    too_many_requests:
      content_type: APPLICATION_JSON
      content: '{ message: "Your request has been blocked due to too many HTTP requests coming from your IP" }'
  returned: Always, as long as the web acl exists
visibility_config:
  description: Visibility config of the web acl
  returned: Always, as long as the web acl exists
  type: dict
  sample:
    cloud_watch_metrics_enabled: true
    metric_name: blub
    sampled_requests_enabled: false
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.wafv2 import compare_priority_rules
from ansible_collections.community.aws.plugins.module_utils.wafv2 import describe_wafv2_tags
from ansible_collections.community.aws.plugins.module_utils.wafv2 import ensure_wafv2_tags
from ansible_collections.community.aws.plugins.module_utils.wafv2 import wafv2_list_web_acls
from ansible_collections.community.aws.plugins.module_utils.wafv2 import wafv2_snake_dict_to_camel_dict


class WebACL:
    def __init__(self, wafv2, name, scope, fail_json_aws):
        self.wafv2 = wafv2
        self.name = name
        self.scope = scope
        self.fail_json_aws = fail_json_aws
        self.existing_acl, self.acl_id, self.locktoken = self.get_web_acl()

    def update(
        self,
        default_action,
        description,
        rules,
        sampled_requests,
        cloudwatch_metrics,
        metric_name,
        custom_response_bodies,
    ):
        req_obj = {
            "Name": self.name,
            "Scope": self.scope,
            "Id": self.acl_id,
            "DefaultAction": default_action,
            "Rules": rules,
            "VisibilityConfig": {
                "SampledRequestsEnabled": sampled_requests,
                "CloudWatchMetricsEnabled": cloudwatch_metrics,
                "MetricName": metric_name,
            },
            "LockToken": self.locktoken,
        }

        if description:
            req_obj["Description"] = description

        if custom_response_bodies:
            req_obj["CustomResponseBodies"] = custom_response_bodies

        try:
            self.wafv2.update_web_acl(**req_obj)
        except (BotoCoreError, ClientError) as e:
            self.fail_json_aws(e, msg="Failed to update wafv2 web acl.")

        self.existing_acl, self.acl_id, self.locktoken = self.get_web_acl()
        return self.existing_acl

    def remove(self):
        try:
            response = self.wafv2.delete_web_acl(
                Name=self.name, Scope=self.scope, Id=self.acl_id, LockToken=self.locktoken
            )
        except (BotoCoreError, ClientError) as e:
            self.fail_json_aws(e, msg="Failed to remove wafv2 web acl.")
        return response

    def get(self):
        if self.existing_acl:
            return self.existing_acl
        return None

    def get_web_acl(self):
        acl_id = None
        locktoken = None
        arn = None
        existing_acl = None
        response = self.list()

        for item in response.get("WebACLs"):
            if item.get("Name") == self.name:
                acl_id = item.get("Id")
                locktoken = item.get("LockToken")
                arn = item.get("ARN")

        if acl_id:
            try:
                existing_acl = self.wafv2.get_web_acl(Name=self.name, Scope=self.scope, Id=acl_id)
            except (BotoCoreError, ClientError) as e:
                self.fail_json_aws(e, msg="Failed to get wafv2 web acl.")
            tags = describe_wafv2_tags(self.wafv2, arn, self.fail_json_aws)
            existing_acl["tags"] = tags
        return existing_acl, acl_id, locktoken

    def list(self):
        return wafv2_list_web_acls(self.wafv2, self.scope, self.fail_json_aws)

    def create(
        self,
        default_action,
        rules,
        sampled_requests,
        cloudwatch_metrics,
        metric_name,
        tags,
        description,
        custom_response_bodies,
    ):
        req_obj = {
            "Name": self.name,
            "Scope": self.scope,
            "DefaultAction": default_action,
            "Rules": rules,
            "VisibilityConfig": {
                "SampledRequestsEnabled": sampled_requests,
                "CloudWatchMetricsEnabled": cloudwatch_metrics,
                "MetricName": metric_name,
            },
        }

        if custom_response_bodies:
            req_obj["CustomResponseBodies"] = custom_response_bodies
        if description:
            req_obj["Description"] = description
        if tags:
            req_obj["Tags"] = ansible_dict_to_boto3_tag_list(tags)

        try:
            self.wafv2.create_web_acl(**req_obj)
        except (BotoCoreError, ClientError) as e:
            self.fail_json_aws(e, msg="Failed to create wafv2 web acl.")

        self.existing_acl, self.acl_id, self.locktoken = self.get_web_acl()
        return self.existing_acl


def format_result(result):
    # We were returning details of the Web ACL inside a "web_acl"  parameter on
    # creation, keep returning it to avoid breaking existing playbooks, but also
    # return what the docs said we return (and returned when no change happened)
    retval = dict(result)
    if "WebACL" in retval:
        retval.update(retval["WebACL"])

    return camel_dict_to_snake_dict(retval, ignore_list=["tags"])


def main():
    arg_spec = dict(
        state=dict(type="str", required=True, choices=["present", "absent"]),
        name=dict(type="str", required=True),
        scope=dict(type="str", required=True, choices=["CLOUDFRONT", "REGIONAL"]),
        description=dict(type="str"),
        default_action=dict(type="str", choices=["Block", "Allow"]),
        rules=dict(type="list", elements="dict"),
        sampled_requests=dict(type="bool", default=False),
        cloudwatch_metrics=dict(type="bool", default=True),
        metric_name=dict(type="str"),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
        custom_response_bodies=dict(type="dict"),
        purge_rules=dict(default=True, type="bool"),
    )

    module = AnsibleAWSModule(
        argument_spec=arg_spec,
        supports_check_mode=True,
        required_if=[["state", "present", ["default_action", "rules"]]],
    )

    state = module.params.get("state")
    name = module.params.get("name")
    scope = module.params.get("scope")
    description = module.params.get("description")
    default_action = module.params.get("default_action")
    rules = module.params.get("rules")
    sampled_requests = module.params.get("sampled_requests")
    cloudwatch_metrics = module.params.get("cloudwatch_metrics")
    metric_name = module.params.get("metric_name")
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")
    purge_rules = module.params.get("purge_rules")
    check_mode = module.check_mode

    custom_response_bodies = module.params.get("custom_response_bodies")
    if custom_response_bodies:
        custom_response_bodies = {}

        for custom_name, body in module.params.get("custom_response_bodies").items():
            custom_response_bodies[custom_name] = snake_dict_to_camel_dict(body, capitalize_first=True)

    if default_action == "Block":
        default_action = {"Block": {}}
    elif default_action == "Allow":
        default_action = {"Allow": {}}

    if rules:
        rules = []
        for rule in module.params.get("rules"):
            rules.append(wafv2_snake_dict_to_camel_dict(snake_dict_to_camel_dict(rule, capitalize_first=True)))

    if not metric_name:
        metric_name = name

    wafv2 = module.client("wafv2")
    web_acl = WebACL(wafv2, name, scope, module.fail_json_aws)
    change = False
    retval = {}

    if state == "present":
        if web_acl.get():
            tags_changed = ensure_wafv2_tags(
                wafv2, web_acl.get().get("WebACL").get("ARN"), tags, purge_tags, module.fail_json_aws, module.check_mode
            )
            change, rules = compare_priority_rules(web_acl.get().get("WebACL").get("Rules"), rules, purge_rules, state)
            change = change or (description and web_acl.get().get("WebACL").get("Description") != description)
            change = change or (default_action and web_acl.get().get("WebACL").get("DefaultAction") != default_action)

            if change and not check_mode:
                retval = web_acl.update(
                    default_action,
                    description,
                    rules,
                    sampled_requests,
                    cloudwatch_metrics,
                    metric_name,
                    custom_response_bodies,
                )
            elif tags_changed:
                retval, acl_id, locktoken = web_acl.get_web_acl()
            else:
                retval = web_acl.get()

            change |= tags_changed

        else:
            change = True
            if not check_mode:
                retval = web_acl.create(
                    default_action,
                    rules,
                    sampled_requests,
                    cloudwatch_metrics,
                    metric_name,
                    tags,
                    description,
                    custom_response_bodies,
                )

    elif state == "absent":
        if web_acl.get():
            if rules:
                if len(rules) > 0:
                    change, rules = compare_priority_rules(
                        web_acl.get().get("WebACL").get("Rules"), rules, purge_rules, state
                    )
                    if change and not check_mode:
                        retval = web_acl.update(
                            default_action,
                            description,
                            rules,
                            sampled_requests,
                            cloudwatch_metrics,
                            metric_name,
                            custom_response_bodies,
                        )
            else:
                change = True
                if not check_mode:
                    retval = web_acl.remove()

    module.exit_json(changed=change, **format_result(retval))


if __name__ == "__main__":
    main()



# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# For the parts taken from the docker inventory script:
# Copyright (c) 2016, Paul Durivage <paul.durivage@gmail.com>
# Copyright (c) 2016, Chris Houseknecht <house@redhat.com>
# Copyright (c) 2016, James Tanner <jtanner@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: docker_containers
short_description: Ansible dynamic inventory plugin for Docker containers
version_added: 1.1.0
author:
  - Felix Fontein (@felixfontein)
extends_documentation_fragment:
  - ansible.builtin.constructed
  - community.docker._docker.api_documentation
  - community.library_inventory_filtering_v1.inventory_filter
description:
  - Reads inventories from the Docker API.
  - Uses a YAML configuration file that ends with V(docker.(yml|yaml\)).
notes:
  - The configuration file must be a YAML file whose filename ends with V(docker.yml) or V(docker.yaml). Other filenames will
    not be accepted.
options:
  plugin:
    description:
      - The name of this plugin, it should always be set to V(community.docker.docker_containers) for this plugin to recognize
        it as its own.
    type: str
    required: true
    choices: [community.docker.docker_containers]

  connection_type:
    description:
      - Which connection type to use the containers.
      - One way to connect to containers is to use SSH (V(ssh)). For this, the options O(default_ip) and O(private_ssh_port)
        are used. This requires that a SSH daemon is running inside the containers.
      - Alternatively, V(docker-cli) selects the P(community.docker.docker#connection) connection plugin, and V(docker-api)
        (default) selects the P(community.docker.docker_api#connection) connection plugin.
      - When V(docker-api) is used, all Docker daemon configuration values are passed from the inventory plugin to the connection
        plugin. This can be controlled with O(configure_docker_daemon).
      - Note that the P(community.docker.docker_api#connection) does B(not work with TCP TLS sockets)!
        See U(https://github.com/ansible-collections/community.docker/issues/605) for more information.
    type: str
    default: docker-api
    choices:
      - ssh
      - docker-cli
      - docker-api

  configure_docker_daemon:
    description:
      - Whether to pass all Docker daemon configuration from the inventory plugin to the connection plugin.
      - Only used when O(connection_type=docker-api).
    type: bool
    default: true
    version_added: 1.8.0

  verbose_output:
    description:
      - Toggle to (not) include all available inspection metadata.
      - Note that all top-level keys will be transformed to the format C(docker_xxx). For example, C(HostConfig) is converted
        to C(docker_hostconfig).
      - If this is V(false), these values can only be used during O(compose), O(groups), and O(keyed_groups).
      - The C(docker) inventory script always added these variables, so for compatibility set this to V(true).
    type: bool
    default: false

  default_ip:
    description:
      - The IP address to assign to ansible_host when the container's SSH port is mapped to interface '0.0.0.0'.
      - Only used if O(connection_type) is V(ssh).
    type: str
    default: 127.0.0.1

  private_ssh_port:
    description:
      - The port containers use for SSH.
      - Only used if O(connection_type) is V(ssh).
    type: int
    default: 22

  add_legacy_groups:
    description:
      - 'Add the same groups as the C(docker) inventory script does. These are the following:'
      - 'C(<container id>): contains the container of this ID.'
      - 'C(<container name>): contains the container that has this name.'
      - 'C(<container short id>): contains the containers that have this short ID (first 13 letters of ID).'
      - 'C(image_<image name>): contains the containers that have the image C(<image name>).'
      - 'C(stack_<stack name>): contains the containers that belong to the stack C(<stack name>).'
      - 'C(service_<service name>): contains the containers that belong to the service C(<service name>).'
      - 'C(<docker_host>): contains the containers which belong to the Docker daemon O(docker_host). Useful if you run this
        plugin against multiple Docker daemons.'
      - 'C(running): contains all containers that are running.'
      - 'C(stopped): contains all containers that are not running.'
      - If this is not set to V(true), you should use keyed groups to add the containers to groups. See the examples for how
        to do that.
    type: bool
    default: false

  filters:
    version_added: 3.5.0
"""

EXAMPLES = """
---
# Minimal example using local Docker daemon
plugin: community.docker.docker_containers
docker_host: unix:///var/run/docker.sock

---
# Minimal example using remote Docker daemon
plugin: community.docker.docker_containers
docker_host: tcp://my-docker-host:2375

---
# Example using remote Docker daemon with unverified TLS
plugin: community.docker.docker_containers
docker_host: tcp://my-docker-host:2376
tls: true

---
# Example using remote Docker daemon with verified TLS and client certificate verification
plugin: community.docker.docker_containers
docker_host: tcp://my-docker-host:2376
validate_certs: true
ca_path: /somewhere/ca.pem
client_key: /somewhere/key.pem
client_cert: /somewhere/cert.pem

---
# Example using constructed features to create groups
plugin: community.docker.docker_containers
docker_host: tcp://my-docker-host:2375
strict: false
keyed_groups:
  # Add containers with primary network foo to a network_foo group
  - prefix: network
    key: 'docker_hostconfig.NetworkMode'
  # Add Linux hosts to an os_linux group
  - prefix: os
    key: docker_platform

---
# Example using SSH connection with an explicit fallback for when port 22 has not been
# exported: use container name as ansible_ssh_host and 22 as ansible_ssh_port
plugin: community.docker.docker_containers
connection_type: ssh
compose:
  ansible_ssh_host: ansible_ssh_host | default(docker_name[1:], true)
  ansible_ssh_port: ansible_ssh_port | default(22, true)

---
# Only consider containers which have a label 'foo', or whose name starts with 'a'
plugin: community.docker.docker_containers
filters:
  # Accept all containers which have a label called 'foo'
  - include: >-
      "foo" in docker_config.Labels
  # Next accept all containers whose inventory_hostname starts with 'a'
  - include: >-
      inventory_hostname.startswith("a")
  # Exclude all containers that did not match any of the above filters
  - exclude: true
"""

import re
import typing as t

from ansible.errors import AnsibleError
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable
from ansible_collections.community.library_inventory_filtering_v1.plugins.plugin_utils.inventory_filter import (
    filter_host,
    parse_filters,
)

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    APIError,
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DOCKER_COMMON_ARGS_VARS,
)
from ansible_collections.community.docker.plugins.plugin_utils._common_api import (
    AnsibleDockerClient,
)
from ansible_collections.community.docker.plugins.plugin_utils._unsafe import (
    make_unsafe,
)

if t.TYPE_CHECKING:
    from ansible.inventory.data import InventoryData
    from ansible.parsing.dataloader import DataLoader


MIN_DOCKER_API = None


class InventoryModule(BaseInventoryPlugin, Constructable):
    """Host inventory parser for ansible using Docker daemon as source."""

    NAME = "community.docker.docker_containers"

    def _slugify(self, value: str) -> str:
        slug = re.sub(r"[^\w-]", "_", value).lower().lstrip("_")
        return f"docker_{slug}"

    def _populate(self, client: AnsibleDockerClient) -> None:
        strict = self.get_option("strict")

        ssh_port = self.get_option("private_ssh_port")
        default_ip = self.get_option("default_ip")
        hostname = self.get_option("docker_host")
        verbose_output = self.get_option("verbose_output")
        connection_type = self.get_option("connection_type")
        add_legacy_groups = self.get_option("add_legacy_groups")

        if self.inventory is None:
            raise AssertionError("Inventory must be there")

        try:
            params = {
                "limit": -1,
                "all": 1,
                "size": 0,
                "trunc_cmd": 0,
                "since": None,
                "before": None,
            }
            containers = client.get_json("/containers/json", params=params)
        except APIError as exc:
            raise AnsibleError(f"Error listing containers: {exc}") from exc

        if add_legacy_groups:
            self.inventory.add_group("running")
            self.inventory.add_group("stopped")

        extra_facts = {}
        if self.get_option("configure_docker_daemon"):
            for option_name, var_name in DOCKER_COMMON_ARGS_VARS.items():
                value = self.get_option(option_name)
                if value is not None:
                    extra_facts[var_name] = value

        filters = parse_filters(self.get_option("filters"))
        for container in containers:
            container_id = container.get("Id")
            short_container_id = container_id[:13]

            try:
                name = container.get("Names", [])[0].lstrip("/")
                full_name = name
            except IndexError:
                name = short_container_id
                full_name = container_id

            facts = {
                "docker_name": make_unsafe(name),
                "docker_short_id": make_unsafe(short_container_id),
            }
            full_facts = {}

            try:
                inspect = client.get_json("/containers/{0}/json", container_id)
            except APIError as exc:
                raise AnsibleError(
                    f"Error inspecting container {name} - {exc}"
                ) from exc

            state = inspect.get("State") or {}
            config = inspect.get("Config") or {}
            labels = config.get("Labels") or {}

            running = state.get("Running")

            groups = []

            # Add container to groups
            image_name = config.get("Image")
            if image_name and add_legacy_groups:
                groups.append(f"image_{image_name}")

            stack_name = labels.get("com.docker.stack.namespace")
            if stack_name:
                full_facts["docker_stack"] = stack_name
                if add_legacy_groups:
                    groups.append(f"stack_{stack_name}")

            service_name = labels.get("com.docker.swarm.service.name")
            if service_name:
                full_facts["docker_service"] = service_name
                if add_legacy_groups:
                    groups.append(f"service_{service_name}")

            ansible_connection = None
            if connection_type == "ssh":
                # Figure out ssh IP and Port
                try:
                    # Lookup the public facing port Nat'ed to ssh port.
                    network_settings = inspect.get("NetworkSettings") or {}
                    port_settings = network_settings.get("Ports") or {}
                    port = port_settings.get(f"{ssh_port}/tcp")[0]  # type: ignore[index]
                except (IndexError, AttributeError, TypeError):
                    port = {}

                try:
                    ip = default_ip if port["HostIp"] == "0.0.0.0" else port["HostIp"]
                except KeyError:
                    ip = ""

                facts.update(
                    {
                        "ansible_ssh_host": ip,
                        "ansible_ssh_port": port.get("HostPort", 0),
                    }
                )
            elif connection_type == "docker-cli":
                facts.update(
                    {
                        "ansible_host": full_name,
                    }
                )
                ansible_connection = "community.docker.docker"
            elif connection_type == "docker-api":
                facts.update(
                    {
                        "ansible_host": full_name,
                    }
                )
                facts.update(extra_facts)
                ansible_connection = "community.docker.docker_api"

            full_facts.update(facts)
            for key, value in inspect.items():
                fact_key = self._slugify(key)
                full_facts[fact_key] = value

            full_facts = make_unsafe(full_facts)

            if ansible_connection:
                for d in (facts, full_facts):
                    if "ansible_connection" not in d:
                        d["ansible_connection"] = ansible_connection

            if not filter_host(self, name, full_facts, filters):
                continue

            if verbose_output:
                facts.update(full_facts)

            self.inventory.add_host(name)
            for group in groups:
                self.inventory.add_group(group)
                self.inventory.add_host(name, group=group)

            for key, value in facts.items():
                self.inventory.set_variable(name, key, value)

            # Use constructed if applicable
            # Composed variables
            self._set_composite_vars(
                self.get_option("compose"), full_facts, name, strict=strict
            )
            # Complex groups based on jinja2 conditionals, hosts that meet the conditional are added to group
            self._add_host_to_composed_groups(
                self.get_option("groups"), full_facts, name, strict=strict
            )
            # Create groups based on variable values and add the corresponding hosts to it
            self._add_host_to_keyed_groups(
                self.get_option("keyed_groups"), full_facts, name, strict=strict
            )

            # We need to do this last since we also add a group called `name`.
            # When we do this before a set_variable() call, the variables are assigned
            # to the group, and not to the host.
            if add_legacy_groups:
                self.inventory.add_group(container_id)
                self.inventory.add_host(name, group=container_id)
                self.inventory.add_group(name)
                self.inventory.add_host(name, group=name)
                self.inventory.add_group(short_container_id)
                self.inventory.add_host(name, group=short_container_id)
                self.inventory.add_group(hostname)
                self.inventory.add_host(name, group=hostname)

                if running is True:
                    self.inventory.add_host(name, group="running")
                else:
                    self.inventory.add_host(name, group="stopped")

    def verify_file(self, path: str) -> bool:
        """Return the possibly of a file being consumable by this plugin."""
        return super().verify_file(path) and path.endswith(
            ("docker.yaml", "docker.yml")
        )

    def _create_client(self) -> AnsibleDockerClient:
        return AnsibleDockerClient(self, min_docker_api_version=MIN_DOCKER_API)

    def parse(
        self,
        inventory: InventoryData,
        loader: DataLoader,
        path: str,
        cache: bool = True,
    ) -> None:
        super().parse(inventory, loader, path, cache)
        self._read_config_data(path)
        client = self._create_client()
        try:
            self._populate(client)
        except DockerException as e:
            raise AnsibleError(f"An unexpected Docker error occurred: {e}") from e
        except RequestException as e:
            raise AnsibleError(
                f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}"
            ) from e



# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import base64
import hashlib
import ipaddress
import json
import re
import time
import typing as t

from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils._acme.errors import (
    ACMEProtocolException,
    ModuleFailException,
    format_error_problem,
)
from ansible_collections.community.crypto.plugins.module_utils._acme.utils import (
    nopad_b64,
)
from ansible_collections.community.crypto.plugins.module_utils._caa import (
    _check_domain_name,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from ansible_collections.community.crypto.plugins.module_utils._acme.acme import (  # pragma: no cover
        ACMEClient,
    )


def create_key_authorization(*, client: ACMEClient, token: str) -> str:
    """
    Returns the key authorization for the given token
    https://tools.ietf.org/html/rfc8555#section-8.1
    """
    accountkey_json = json.dumps(
        client.account_jwk, sort_keys=True, separators=(",", ":")
    )
    thumbprint = nopad_b64(hashlib.sha256(accountkey_json.encode("utf8")).digest())
    return f"{token}.{thumbprint}"


def combine_identifier(*, identifier_type: str, identifier: str) -> str:
    return f"{identifier_type}:{identifier}"


def normalize_combined_identifier(identifier: str) -> str:
    identifier_type, identifier = split_identifier(identifier)
    # Normalize DNS names and IPs
    identifier = identifier.lower()
    return combine_identifier(identifier_type=identifier_type, identifier=identifier)


def split_identifier(identifier: str) -> tuple[str, str]:
    parts = identifier.split(":", 1)
    if len(parts) != 2:
        raise ModuleFailException(
            f'Identifier "{identifier}" is not of the form <type>:<identifier>'
        )
    return parts[0], parts[1]


_Challenge = t.TypeVar("_Challenge", bound="Challenge")


class Challenge:
    def __init__(self, *, data: dict[str, t.Any], url: str) -> None:
        self.data = data

        self.type: str = data["type"]
        self.url = url
        self.status: str = data["status"]
        self.token: str | None = data.get("token")

    @classmethod
    def from_json(
        cls: type[_Challenge],
        *,
        client: ACMEClient,
        data: dict[str, t.Any],
        url: str | None = None,
    ) -> _Challenge:
        return cls(data=data, url=url or data["url"])

    def call_validate(self, client: ACMEClient) -> None:
        challenge_response: dict[str, t.Any] = {}
        client.send_signed_request(
            self.url,
            challenge_response,
            error_msg="Failed to validate challenge",
            expected_status_codes=[200, 202],
        )

    def to_json(self) -> dict[str, t.Any]:
        return self.data.copy()

    def get_validation_data(
        self, *, client: ACMEClient, identifier_type: str, identifier: str
    ) -> dict[str, t.Any] | None:
        if self.type == "dns-persist-01":
            # https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-01.html#section-3.1
            account_uri = self.data.get("accounturi")
            issuer_domain_names = self.data.get("issuer-domain-names")
            if account_uri is None:
                # In version 00 of the draft, accounturi isn't present.
                # Since that's what Pebble currently implements,
                # let's fake the value if we have it.
                # (https://www.ietf.org/archive/id/draft-ietf-acme-dns-persist-00.html#section-6)
                account_uri = client.account_uri
            if (
                not isinstance(account_uri, str)
                or not isinstance(issuer_domain_names, list)
                or not all(isinstance(idn, str) for idn in issuer_domain_names)
            ):
                return None
            if not (1 <= len(issuer_domain_names) <= 10):
                client.module.warn(
                    f"The dns-persist-01 challenge for DNS:{identifier} has {len(issuer_domain_names)}"
                    " issuer domain names, which is not in [1, 10]. Ignoring malformed challenge."
                )
                return None
            for idn in issuer_domain_names:
                try:
                    _check_domain_name(idn)
                    if idn != idn.lower() or len(idn) > 253:
                        raise ValueError()
                except ValueError:
                    client.module.warn(
                        f"The dns-persist-01 challenge for DNS:{identifier} has an invalid"
                        f" issuer domain name {idn!r}. Ignoring malformed challenge."
                    )
                    return None
            return {
                "account_uri": account_uri,
                "issuer_domain_names": issuer_domain_names,
            }

        if self.token is None:
            return None

        token = re.sub(r"[^A-Za-z0-9_\-]", "_", self.token)
        key_authorization = create_key_authorization(client=client, token=token)

        if self.type == "http-01":
            # https://tools.ietf.org/html/rfc8555#section-8.3
            return {
                "resource": f".well-known/acme-challenge/{token}",
                "resource_value": key_authorization,
            }

        if self.type == "dns-01":
            if identifier_type != "dns":
                return None
            # https://tools.ietf.org/html/rfc8555#section-8.4
            resource = "_acme-challenge"
            value = nopad_b64(hashlib.sha256(to_bytes(key_authorization)).digest())
            record = f"{resource}.{identifier[2:] if identifier.startswith('*.') else identifier}"
            return {
                "resource": resource,
                "resource_value": value,
                "record": record,
            }

        if self.type == "dns-account-01":
            if identifier_type != "dns" or client.account_uri is None:
                return None
            # https://datatracker.ietf.org/doc/html/draft-ietf-acme-dns-account-label-02#section-3.2
            prefix = (
                base64.b32encode(
                    hashlib.sha256(client.account_uri.encode("utf8")).digest()[:10]
                )
                .decode("ascii")
                .lower()
            )
            resource = f"_{prefix}._acme-challenge"
            value = nopad_b64(hashlib.sha256(to_bytes(key_authorization)).digest())
            record = f"{resource}.{identifier[2:] if identifier.startswith('*.') else identifier}"
            return {
                "resource": resource,
                "resource_value": value,
                "record": record,
            }

        if self.type == "tls-alpn-01":
            # https://www.rfc-editor.org/rfc/rfc8737.html#section-3
            if identifier_type == "ip":
                # IPv4/IPv6 address: use reverse mapping (RFC1034, RFC3596)
                resource = ipaddress.ip_address(identifier).reverse_pointer
                if not resource.endswith("."):
                    resource += "."
            else:
                resource = identifier
            b_value = base64.b64encode(
                hashlib.sha256(to_bytes(key_authorization)).digest()
            )
            return {
                "resource": resource,
                "resource_original": combine_identifier(
                    identifier_type=identifier_type, identifier=identifier
                ),
                "resource_value": b_value,
            }

        # Unknown challenge type: ignore
        return None


_Authorization = t.TypeVar("_Authorization", bound="Authorization")


class Authorization:
    def __init__(self, *, url: str) -> None:
        self.url = url

        self.data: dict[str, t.Any] | None = None
        self.challenges: list[Challenge] = []
        self.status: str | None = None
        self.identifier_type: str | None = None
        self.identifier: str | None = None

    def _setup(self, *, client: ACMEClient, data: dict[str, t.Any]) -> None:
        data["uri"] = self.url
        self.data = data
        # While 'challenges' is a required field, apparently not every CA cares
        # (https://github.com/ansible-collections/community.crypto/issues/824)
        if data.get("challenges"):
            self.challenges = [
                Challenge.from_json(client=client, data=challenge)
                for challenge in data["challenges"]
            ]
        else:
            self.challenges = []
        self.status = data["status"]
        self.identifier = data["identifier"]["value"]
        self.identifier_type = data["identifier"]["type"]
        if data.get("wildcard", False):
            self.identifier = f"*.{self.identifier}"

    @classmethod
    def from_json(
        cls: type[_Authorization],
        *,
        client: ACMEClient,
        data: dict[str, t.Any],
        url: str,
    ) -> _Authorization:
        result = cls(url=url)
        result._setup(client=client, data=data)
        return result

    @classmethod
    def from_url(
        cls: type[_Authorization], *, client: ACMEClient, url: str
    ) -> _Authorization:
        result = cls(url=url)
        result.refresh(client=client)
        return result

    @classmethod
    def create(
        cls: type[_Authorization],
        *,
        client: ACMEClient,
        identifier_type: str,
        identifier: str,
    ) -> _Authorization:
        """
        Create a new authorization for the given identifier.
        Return the authorization object of the new authorization
        https://tools.ietf.org/html/draft-ietf-acme-acme-02#section-6.4
        """
        new_authz = {
            "identifier": {
                "type": identifier_type,
                "value": identifier,
            },
        }
        if "newAuthz" not in client.directory.directory:
            raise ACMEProtocolException(
                module=client.module,
                msg="ACME endpoint does not support pre-authorization",
            )
        url = client.directory["newAuthz"]

        result, info = client.send_signed_request(
            url,
            new_authz,
            error_msg="Failed to request challenges",
            expected_status_codes=[200, 201],
        )
        if not isinstance(result, dict):
            raise ACMEProtocolException(
                module=client.module,
                msg="Unexpected authorization creation result",
                content_json=result,
            )
        return cls.from_json(client=client, data=result, url=info["location"])

    @property
    def combined_identifier(self) -> str:
        if self.identifier_type is None or self.identifier is None:
            raise ValueError("Data not present")
        return combine_identifier(
            identifier_type=self.identifier_type, identifier=self.identifier
        )

    def to_json(self) -> dict[str, t.Any]:
        if self.data is None:
            raise ValueError("Data not present")
        return self.data.copy()

    def refresh(self, *, client: ACMEClient) -> bool:
        result, info = client.get_request(self.url)
        if not isinstance(result, dict):
            raise ACMEProtocolException(
                module=client.module,
                msg="Unexpected authorization data",
                info=info,
                content_json=result,
            )
        changed = self.data != result
        self._setup(client=client, data=result)
        return changed

    def get_challenge_data(self, *, client: ACMEClient) -> dict[str, t.Any]:
        """
        Returns a dict with the data for all proposed (and supported) challenges
        of the given authorization.
        """
        if self.identifier_type is None or self.identifier is None:
            raise ValueError("Data not present")
        data = {}
        for challenge in self.challenges:
            validation_data = challenge.get_validation_data(
                client=client,
                identifier_type=self.identifier_type,
                identifier=self.identifier,
            )
            if validation_data is not None:
                data[challenge.type] = validation_data
        return data

    def get_error_details(self) -> str | None:
        error_details = []
        for challenge in self.challenges:
            if challenge.status == "invalid":
                msg = f"Challenge {challenge.type}"
                if "error" in challenge.data:
                    problem = format_error_problem(
                        challenge.data["error"],
                        subproblem_prefix=f"{challenge.type}.",
                    )
                    msg = f"{msg}: {problem}"
                error_details.append(msg)
        return "; ".join(error_details) if error_details else None

    def raise_error(self, *, error_msg: str, module: AnsibleModule) -> t.NoReturn:
        """
        Aborts with a specific error for a challenge.
        """
        error_details = self.get_error_details()
        error_details_str = f" {error_details}" if error_details else ""
        raise ACMEProtocolException(
            module=module,
            msg=f"Failed to validate challenge for {self.combined_identifier}: {error_msg}.{error_details_str}",
            extras={
                "identifier": self.combined_identifier,
                "authorization": self.data,
            },
        )

    def find_challenge(self, *, challenge_type: str) -> Challenge | None:
        for challenge in self.challenges:
            if challenge_type == challenge.type:
                return challenge
        return None

    def wait_for_validation(self, *, client: ACMEClient) -> bool:
        while True:
            self.refresh(client=client)
            if self.status in ["valid", "invalid", "revoked"]:
                break
            time.sleep(2)

        if self.status == "invalid":
            self.raise_error(error_msg='Status is "invalid"', module=client.module)

        return self.status == "valid"

    def call_validate(
        self, *, client: ACMEClient, challenge_type: str, wait: bool = True
    ) -> bool:
        """
        Validate the authorization provided in the auth dict. Returns True
        when the validation was successful and False when it was not.
        """
        challenge = self.find_challenge(challenge_type=challenge_type)
        if challenge is None:
            raise ModuleFailException(
                f'Found no challenge of type "{challenge_type}" for identifier {self.combined_identifier}!'
            )

        challenge.call_validate(client)

        if not wait:
            return self.status == "valid"
        return self.wait_for_validation(client=client)

    def can_deactivate(self) -> bool:
        """
        Deactivates this authorization.
        https://community.letsencrypt.org/t/authorization-deactivation/19860/2
        https://tools.ietf.org/html/rfc8555#section-7.5.2
        """
        return self.status in ("valid", "pending")

    def is_in_final_state(self, *, allow_valid: bool = True) -> bool:
        if allow_valid and self.status == "valid":
            return True
        return self.status in ("invalid", "revoked", "deactivated", "expired")

    def deactivate(self, *, client: ACMEClient) -> bool | None:
        """
        Deactivates this authorization.
        https://community.letsencrypt.org/t/authorization-deactivation/19860/2
        https://tools.ietf.org/html/rfc8555#section-7.5.2
        """
        if not self.can_deactivate():
            return None
        authz_deactivate = {"status": "deactivated"}
        result, info = client.send_signed_request(
            self.url, authz_deactivate, fail_on_error=False
        )
        if 200 <= info["status"] < 300 and isinstance(result, dict):
            self._setup(client=client, data=result)
            return self.status == "deactivated"
        return False

    @classmethod
    def deactivate_url(
        cls: type[_Authorization], *, client: ACMEClient, url: str
    ) -> _Authorization:
        """
        Deactivates this authorization.
        https://community.letsencrypt.org/t/authorization-deactivation/19860/2
        https://tools.ietf.org/html/rfc8555#section-7.5.2
        """
        authz = cls(url=url)
        authz_deactivate = {"status": "deactivated"}
        result, _info = client.send_signed_request(
            url, authz_deactivate, fail_on_error=True
        )
        if not isinstance(result, dict):
            raise ACMEProtocolException(
                module=client.module,
                msg="Unexpected challenge deactivation result",
                content_json=result,
            )
        authz._setup(client=client, data=result)
        return authz


def wait_for_validation(
    *, authzs: t.Iterable[Authorization], client: ACMEClient
) -> None:
    """
    Wait until a list of authz is valid. Fail if at least one of them is invalid or revoked.
    """
    while authzs:
        authzs_next = []
        for authz in authzs:
            authz.refresh(client=client)
            if authz.status in ["valid", "invalid", "revoked"]:
                if authz.status != "valid":
                    authz.raise_error(
                        error_msg='Status is not "valid"', module=client.module
                    )
            else:
                authzs_next.append(authz)
        if authzs_next:
            time.sleep(2)
        authzs = authzs_next


__all__ = (
    "Authorization",
    "Challenge",
    "combine_identifier",
    "create_key_authorization",
    "normalize_combined_identifier",
    "split_identifier",
    "wait_for_validation",
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Tim Cragg (@timcragg)
# Copyright: (c) 2025, Shreyas Srish (@shrsr)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l4l7_service_graph_template_node
version_added: "2.12.0"
short_description: Manage L4-L7 Service Graph Templates Nodes (vns:AbsNode)
description:
- Manage Manage L4-L7 Service Graph Templates Nodes.
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  service_graph:
    description:
    - The name of an existing Service Graph.
    type: str
  description:
    description:
    - The description of the Service Graph Template Node.
    type: str
  node:
    description:
    - The name of the Service Graph Template Node.
    type: str
  functional_template_type:
    description:
    - The functional template type for the node.
    - The APIC defaults to C(other) when unset during creation.
    type: str
    choices: [
      adc_one_arm,
      adc_two_arm,
      cloud_native_fw,
      cloud_native_lb,
      cloud_vendor_fw,
      cloud_vendor_lb,
      fw_routed,
      fw_trans,
      other
    ]
    aliases: [ func_template_type ]
  function_type:
    description:
    - The type of function.
    - The APIC defaults to C(go_to) when unset during creation.
    type: str
    choices: [ go_to, go_through, l1, l2 ]
    aliases: [ func_type ]
  device:
    description:
    - The name of an existing logical device.
    type: str
  device_tenant:
    description:
    - The tenant the logical device exists under.
    - This variable is only used if logical device and node exist within different tenants.
    - Intended use case is when the device is in the C(common) tenant but the node is not.
    type: str
  managed:
    description:
    - Whether this device managed by the apic.
    - The APIC defaults to C(true) when unset during creation.
    type: bool
  routing_mode:
    description:
    - The routing mode for the node.
    - The APIC defaults to C(unspecified) when unset during creation.
    type: str
    choices: [ redirect, unspecified ]
  is_copy:
    description:
    - Whether the device is a copy device.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  share_encap:
    description:
    - Whether to share encapsulation across the service graph.
    - The APIC defaults to C(false) when unset during creation.
    type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner
notes:
- The I(tenant), I(service_graph) and I(device) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l4l7_service_graph_template) and M(cisco.aci.aci_l4l7_device) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l4l7_service_graph_template
- module: cisco.aci.aci_l4l7_device
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(vnsAbsNode)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Shreyas Srish (@shrsr)
"""

EXAMPLES = r"""
- name: Add a new Service Graph Template Node
  cisco.aci.aci_l4l7_service_graph_template_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: test-graph
    node: test-node
    functional_template_type: adc_one_arm
    function_type: GoTo
    device: test-device
    managed: false
    routing_mode: redirect
    state: present
  delegate_to: localhost

- name: Query a Service Graph Template Node
  cisco.aci.aci_l4l7_service_graph_template_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: test-graph
    node: test-node
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Service Graph Template Nodes
  cisco.aci.aci_l4l7_service_graph_template_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a Service Graph Template Node
  cisco.aci.aci_l4l7_service_graph_template_node:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    service_graph: test-graph
    node: test-node
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import L4L7_FUNC_TYPES_MAPPING, L4L7_FUNCTIONAL_TEMPLATE_TYPES_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        service_graph=dict(type="str"),
        node=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        functional_template_type=dict(type="str", aliases=["func_template_type"], choices=list(L4L7_FUNCTIONAL_TEMPLATE_TYPES_MAPPING)),
        function_type=dict(type="str", aliases=["func_type"], choices=list(L4L7_FUNC_TYPES_MAPPING)),
        device=dict(type="str"),
        device_tenant=dict(type="str"),
        managed=dict(type="bool"),
        routing_mode=dict(type="str", choices=["redirect", "unspecified"]),
        is_copy=dict(type="bool"),
        description=dict(type="str"),
        share_encap=dict(type="bool"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "service_graph", "node"]],
            ["state", "present", ["tenant", "service_graph", "node", "device"]],
        ],
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    service_graph = module.params.get("service_graph")
    node = module.params.get("node")
    state = module.params.get("state")
    functional_template_type = L4L7_FUNCTIONAL_TEMPLATE_TYPES_MAPPING.get(module.params.get("functional_template_type"))
    function_type = L4L7_FUNC_TYPES_MAPPING.get(module.params.get("function_type"))
    device = module.params.get("device")
    device_tenant = module.params.get("device_tenant")
    managed = aci.boolean(module.params.get("managed"))
    routing_mode = "Redirect" if module.params.get("routing_mode") == "redirect" else module.params.get("routing_mode")
    is_copy = aci.boolean(module.params.get("is_copy"))
    description = module.params.get("description")
    share_encap = aci.boolean(module.params.get("share_encap"))

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="vnsAbsGraph",
            aci_rn="AbsGraph-{0}".format(service_graph),
            module_object=service_graph,
            target_filter={"name": service_graph},
        ),
        subclass_2=dict(
            aci_class="vnsAbsNode",
            aci_rn="AbsNode-{0}".format(node),
            module_object=node,
            target_filter={"name": node},
        ),
        child_classes=["vnsRsNodeToLDev"],
    )

    aci.get_existing()
    if not device_tenant:
        device_tenant = tenant
    dev_tdn = "uni/tn-{0}/lDevVip-{1}".format(device_tenant, device)

    if state == "present":
        aci.payload(
            aci_class="vnsAbsNode",
            class_config=dict(
                name=node,
                funcTemplateType=functional_template_type,
                funcType=function_type,
                managed=managed,
                routingMode=routing_mode,
                isCopy=is_copy,
                descr=description,
                shareEncap=share_encap,
            ),
            child_configs=[
                dict(
                    vnsRsNodeToLDev=dict(
                        attributes=dict(tDn=dev_tdn),
                    ),
                ),
            ],
        )
        aci.get_diff(aci_class="vnsAbsNode")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    # Standard files documentation fragment
    DOCUMENTATION = r"""
description:
  - The module allows to set, modify and delete multiple DNS record sets at once.
  - With the O(prune) option, it is also possible to delete existing record sets that are not mentioned in the module parameters.
    With this, it is possible to synchronize the expected state of a DNS zone with the expected state.
  - It is possible to ignore certain record sets by specifying O(record_sets[].ignore=true) for that record set.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  idempotent:
    support: full
options:
  zone_name:
    description:
      - The DNS zone to modify.
      - Exactly one of O(zone_name) and O(zone_id) must be specified.
    type: str
    aliases:
      - zone
  zone_id:
    description:
      - The ID of the DNS zone to modify.
      - Exactly one of O(zone_name) and O(zone_id) must be specified.
  prune:
    description:
      - If set to V(true), will remove all existing records in the zone that are not listed in O(record_sets).
    type: bool
    default: false
  record_sets:
    description:
      - The records that should be present in the zone.
    required: true
    type: list
    elements: dict
    aliases:
      - records
    suboptions:
      # (The following must be kept in sync with the equivalent lines in <provider_name>.py!)
      record:
        description:
          - The full DNS record to create or delete.
          - Exactly one of O(record_sets[].record) and O(record_sets[].prefix) must be specified.
        type: str
      prefix:
        description:
          - The prefix of the DNS record.
          - This is the part of O(record_sets[].record) before O(zone_name). For example, if the record to be modified is
            C(www.example.com) for the zone C(example.com), the prefix is V(www). If the record in this example would be C(example.com),
            the prefix would be V('') (empty string).
          - Exactly one of O(record_sets[].record) and O(record_sets[].prefix) must be specified.
        type: str
      ttl:
        description:
          - The TTL to give the new record, in seconds.
        type: int
      type:
        description:
          - The type of DNS record to create or delete.
        required: true
        type: str
      value:
        description:
          - The new value when creating a DNS record.
          - YAML lists or multiple comma-spaced values are allowed.
          - When deleting a record all values for the record must be specified or it will not be deleted.
          - Must be specified if O(record_sets[].ignore=false).
        type: list
        elements: str
      ignore:
        description:
          - If set to V(true), O(record_sets[].value) will be ignored.
          - This is useful when O(prune=true), but you do not want certain entries to be removed without having to know their
            current value.
        type: bool
        default: false
"""
